;******************************************************************************
;
;                              E V 1 3 . 1 7 2
;
;                             E Q U A T E S . H
;
;                             1 0   A p r   9 6
;
;******************************************************************************

ADCTL:          EQU     30H               ;A/D CONTROL/STATUS REGISTER
ADR1:           EQU     31H               ;A/D RESULT REGISTER 1
ADR2:           EQU     32H               ;A/D RESULT REGISTER 2
ADR3:           EQU     33H               ;A/D RESULT REGISTER 3
ADR4:           EQU     34H               ;A/D RESULT REGISTER 4
BAUD:           EQU     2BH               ;SCI BAUD RATE CONTROL REGISTER
BPROT:          EQU     35H               ;BLOCK PROTECT REGISTER
CFORC:          EQU     0BH               ;TIMER COMPARE FORCE REGISTER
CONFIG:         EQU     3FH               ;CONFIGURATION CONTROL REGISTER
COPRST:         EQU     3AH               ;ARM/RESET COP TIMER CIRCUITRY
DDRC:           EQU     07H               ;DATA DIRECTION REG FOR PORT C
DDRD:           EQU     09H               ;DATA DIRECTION REG FOR PORT D
FIVE_K:         EQU     5000              ;5,000 decimal
HPRIO:          EQU     3CH               ;HIGHEST PRIORITY INTERRUPT
INIT:           EQU     3DH               ;RAM AND I/O MAPPING REGISTER
MODE0:          EQU     00H               ;Symbolic for Mode 0, Power-up Delay
LMODE:          EQU     01H               ;Symbolic for Mode 1, Level Mode
MODE1:          EQU     01H               ;Symbolic for Mode 1, Const. Current
MODE2:          EQU     02H               ;Symbolic for Mode 2, Const. Voltage
MODE3:          EQU     03H               ;Symbolic for Mode 3, Const. Current
MODE4:          EQU     04H               ;Symbolic for Mode 4, Const. Voltage
MODE8:          EQU     08H               ;Symbolic for Mode 8, EPTI Fault Mode
MODE32:         EQU     20H               ;Symbolic for Mode 32, EPTI Off Mode
MODE100:        EQU     64H               ;Symbolic for Mode 100, Badicheq Wait
MODE128:        EQU     80H               ;Symbolic for Mode 128, EPTI Code
MODE200:        EQU     0C8H              ;Symbolic for Mode 200, Wait on EPTI
MODE254:        EQU     0FEH              ;Symbolic for Mode 254, Completion
MODE255:        EQU     0FFH              ;Symbolic for Mode 255, Fault Mode
OC1D:           EQU     0DH               ;OUTPUT COMPARE 1 DATA REGISTER
OC1M:           EQU     0CH               ;OUTPUT COMPARE 1 MASK REGISTER
OPTION:         EQU     39H               ;SYSTEM CONFIGURATION OPTIONS
PACNT:          EQU     27H               ;PULSE ACCUMULATOR COUNT REG
PACTL:          EQU     26H               ;PULSE ACCUMULATOR CONTROL REG
PIOC:           EQU     02H               ;PARALLEL I/O CONTROL REGISTER
PORTA:          EQU     00H               ;PORT A DATA REGISTER
PORTB:          EQU     04H               ;PORT B DATA REGISTER
PORTC:          EQU     03H               ;PORT C DATA REGISTER
PORTCL:         EQU     05H               ;PORT C LATCHED DATA REGISTER
PORTD:          EQU     08H               ;PORT D DATA REGISTER
PORTE:          EQU     0AH               ;PORT E DATA REGISTER
PPROG:          EQU     103BH             ;EEPROM PROGRAMMING REGISTER
REGBAS:         EQU     1000H             ;REGISTER BASE ADDRESS
RST:            EQU     020h              ;DLCS *RST pin
SCCR1:          EQU     2CH               ;SCI CONTROL REGISTER 1
SCCR2:          EQU     2DH               ;SCI CONTROL REGISTER 2
SCDR:           EQU     2FH               ;SCI DATA REGISTER
SCSR:           EQU     2EH               ;SCI STATUS REGIST+ER
SPCR:           EQU     28H               ;SPI CONTROL REGISTER
SPDR:           EQU     2AH               ;SPI DATA REGISTER
SPSR:           EQU     29H               ;SPI STATUS REGISTER
TCNT:           EQU     0EH               ;TIMER COUNTER REGISTER
TCTL1:          EQU     20H               ;TIMER CONTROL REGISTER 1
TCTL2:          EQU     21H               ;TIMER CONTROL REGISTER 2
TEN:            EQU     0AH               ;TO GET AMPERAGE DOWN TO 1/10 AMP
TEST1:          EQU     3EH               ;FACTORY TEST REGISTER
TFLG1:          EQU     23H               ;MAIN TIMER INT. FLAG REG 1
TFLG2:          EQU     25H               ;MAIN TIMER INT. FLAG REG 2
TIC1:           EQU     10H               ;TIMER INPUT CAPTURE REGISTER 1
TIC2:           EQU     12H               ;TIMER INPUT CAPTURE REGISTER 2
TIC3:           EQU     14H               ;TIMER INPUT CAPTURE REGISTER 3
TMSK1:          EQU     22H               ;MAIN TIMER INT MASK REGISTER 1
TMSK2:          EQU     24H               ;MAIN TIMER INT MASK REGISTER 2
TOC1:           EQU     16H               ;TIMER OUTPUT COMPARE REG 1
TOC2:           EQU     18H               ;TIMER OUTPUT COMPARE REG 2
TOC3:           EQU     1AH               ;TIMER OUTPUT COMPARE REG 3
TOC4:           EQU     1CH               ;TIMER OUTPUT COMPARE REG 4
TI4O5:          EQU     1EH               ;TIMER OUTPUT COMPARE REG 5 OR IC 4
TOC5:           EQU     1EH               ;TIMER OUTPUT COMPARE REG 5 OR IC 4

FIVE_V_TO_LSB:  EQU     0c4h              ;(5v / 255 lsb) * 10000
                                          ;.0196v/lsb = 5v / 255 lsb
                                          ;5 = 5.0 volts max to the A/D channel
                                          ;255 = # of lsb's available
                                          ;10,000 to keep us as an integer

                CPU     "68HC11.TBL"      ; LOAD 6811 TABLE
                HOF     "BIN8"            ; Binary 8 Bit Hex


;******************************************************************************
;  These constants are used to help the Evxxx.EXE diagnostic program decode the
;  voltage and current readings.  Do not change these values.
;
VOLTS_MULTIPLY: equ     10000             ;Multiplier for Voltage_Scale
AMPS_MULTIPLY:  equ     100               ;Multiplier for Current_Scale

;******************************************************************************
;  This constant is used by the Temperature Compensation routine to determine
;  the required offset voltage for the present temperature.  The value
;  represented is the number of LSBs per degree Kelvin multiplied by 100 to
;  obtain an integer value.  Do not change this label.  See the temperature
;  compensation routine in the .asm file for details on how this number was
;  determined.  Do not change this value.
;
LSB_PER_KELVIN: equ     211               ;Lsb per degree Kelvin * 100

;******************************************************************************
;  The Auxiliary Battery voltage is monitored to ensure the aux battery energy
;  is not being depleted as well as not exceeding a maximum limit.  The
;  following constants define the Auxiliary Battery Minimum voltage and the
;  Auxiliary Battery Maximum voltage reported to A/D channel AN4.  Multiply the
;  desired value by 10,000 to ensure an integer is provided to the assembler.
;  The 10,000 is cancelled by the FIVE_V_TO_LSB divisor since it too was multiplied
;  by 10,000.  Do not change these values.
;
V_AUX_BATT_MIN: equ     {24500/FIVE_V_TO_LSB}   ;2.45 vdc min (11.5) on A/D AN4
                                                ;24500 = 2.45 * 10000

V_AUX_BATT_MON: equ     {26700/FIVE_V_TO_LSB}   ;2.67 vdc min (12.4) on A/D AN4
                                                ;26700 = 2.67 * 10000

V_AUX_BATT_MAX: equ     {31900/FIVE_V_TO_LSB}   ;3.19 vdc max (13.5) on A/D AN4
                                                ;31900 = 3.19 * 10000

;******************************************************************************
;  The Dolphin Unit is monitored for faults based on the voltage present on
;  the Precharge mnius (pin EE) of of the Dolphin unit.  The Dolphin failures
;  being monitored by A/D channel AN5 are as follows:
;
;  Dolphin Precharge Minus line when measured less than V_DolphinNorm value is
;  considered to be in a normal state.  Multiply the desired value by 10,000 to
;  ensure an integer is provided to the assembler.  The 10,000 is cancelled by
;  the FIVE_V_TO_LSB divisor since it too was multiplied by 10,000.  Do not change
;  this value.
;
V_DOLPHINNORM:  equ     {980/FIVE_V_TO_LSB}     ; .098vdc maximum on A/D PE5
                                                ;980 = .098 * 10000

;  Dolphin Precharge Minus line when measured less than DolphinFetHi and
;  greater than DolphinFetLo is considered to be a fault.  The Dolphin unit is
;  shut down and the Dolphin Fet is OFF.  Do not charge.  Multiply the desired
;  value by 10,000 to ensure an integer is provided to the assembler.  The
;  10,000 is cancelled by the FIVE_V_TO_LSB divisor since it too was multiplied by
;  10,000.  Do not change this value.
;
V_DOLPHINFETHI: equ     {33400/FIVE_V_TO_LSB}   ;3.340vdc maximum on A/D PE5
                                                ;33400 = 3.34 * 10000

V_DOLPHINFETLO: equ     {24500/FIVE_V_TO_LSB}   ;2.450vdc maximum on A/D PE5
                                                ;24500 = 2.45 * 10000

;  Dolphin Precharge Minus line when measured less than DolphinLidHi and
;  greater than DolphinLidLo is considered to be a fault.  The Dolphin unit is
;  shut down and the Dolphin Fet is OFF.  Do not charge.  Multiply the desired
;  value by 10,000 to ensure an integer is provided to the assembler.  The
;  10,000 is cancelled by the FIVE_V_TO_LSB divisor since it too was multiplied by
;  10,000.  Do not change this value.
;
V_DOLPHINLIDHI: equ     {18000/FIVE_V_TO_LSB}   ;1.800vdc maximum on A/D PE5
                                                ;18000 = 1.80 * 10000

V_DOLPHINLIDLO: equ     {8400/FIVE_V_TO_LSB}    ; .840vdc minimum on A/D PE5
                                                ;8400 = 0.84 * 10000

;******************************************************************************
;  This constant is used to set the maximum limit when calculating the power
;  command, which the charge command (ch_command) is derived from.  Do not
;  change this limit.  See the comments in the CONTROL: module within the .asm
;  file for more information.  Formerly called the CC_LIM and CV_LIM
;  parameters.
;
OVERFLOW_LIMIT: equ     1000h             ;16-bit 'power' overflow limit
                                          ; cv & cc = 2^(8 + CV_N)

;******************************************************************************
;  These constants defines how many samples per hour are being taken to
;  calculate the amount of energy being returned to the battery pack.
;
TSAMPLE:        equ     1                 ;We're sampling once a minute.
                                          ;Must be an integer number 1 to 60.
                                          ;NOTE:  >> Leave at 1 for now. <<

SAMPLES_HOUR:   equ     {60/Tsample}      ;Number of samples that will be taken
                                          ;  per hour (60 mins / Tsample)
                                          ;  i.e. Tsample = 2 so you'll be
                                          ;  taking 30 samples per hour.
                                          ;NOTE:  >> Leave at 60 for now. <<

;******************************************************************************
;  This constant is used to determine the scale factor when scaling the power
;  command to derive the charge command (ch_command).  The number used here
;  represents the number of right shifts the D accumulator will perform,
;  yielding the new charge command.  See the comments in the CONTROL: and CTRL:
;  modules within the .asm file for more information.  Formerly called the CV_N
;  and CC_N parameters.
;
POWER_SCALE:    equ     4                 ;cc and cv gain scale factor exponent
                                          ;scale factor = 2^n

;******************************************************************************
;
;                        E N D   O F   E Q U A T E S . H
;
;******************************************************************************

