;Don't change the following information !

                TITL    "EV13.172 Copyright (c) Hughes Power Control Systems 1996 - Unpublished Work - All Rights Reserved - HUGHES PROPRIETARY"
                PAGE    76                ;Lines per printed page

;Don't change the above information!

;  Ev13.172
;
;  Now operates with 4 modes of constant voltage, as follows:
;     Mode 1 - Normal, Constant Voltage operation.  Transitions to Mode 2.
;     Mode 2 - Normal, Constant Voltage operation.  Transitions to Mode 3
;              if LEVEL_AFTER has been reached, otherwise to Completion Mode.
;     Mode 3 - Traction Pack leveling, mandatory if the number of charge
;              complete cycles exceeds the number coded in LEVEL_AFTER found in
;              the customer.h file.  Transitions to Completion Mode.
;     Mode 4 - Auxiliary Battery charging only, Constant Voltage.  Transitions
;              to Completion Mode.
;
;     All modes MUST have IMAX_MODEn set to a value greater than 0 or else the
;     mode will not charge.
;
;  During initialization, if the EE_CYCLE_CNT is found to be at 0, a mandatory
;  traction pack leveling (Mode 3) will occur.  Modes 1 and 2 will be bypassed
;  for entry into Mode 3.
;
;  Recovered EEPROM from Oc1_Irq.
;
;  Recovered EEPROM from Ram_Out.
;
;  Removed amp hour trip outs for mode 1 & 2.
;
;  Removed all code pertaining to Constant Current modes of operation.
;
;  This code was based on Ev13.142 dated 11 Mar 96.  All previous comments have
;  been removed
;
;  12 Apr 96   JWH
;******************************************************************************

                incl    "equates.h"

;******************************************************************************
;  This is where the Ram storage starts.
;
                org     0h                ; Ram data area
;
                incl    "customer.h"      ; Customer Modifiable Variables
                incl    "ramequs.h"       ; Defined Ram Storage Variables

;******************************************************************************
;  Startup calculations
;
;  This constant determines the maximum number of amp mins allowed to be
;  applied to the battery pack before entering fault mode.
;
MAX_AMP_MINS:   equ     AMPHR_MAXLIMIT*60 ;60 mins per hr * Max Amp Hours

;******************************************************************************


;****************************************************************************
;  RESET (initialization)
;  Description:  Sets 68HC11E2 registers to known states.   
;  Prog flow:    Code is executed on reset, jumps to main on completion.
;  Regs Altered: a, s, x
;  Global Read:  None.
;  Global Write: None.
;  Passed Value: None.
;  Return value: None.
;
;  Note: On RESET the following HC11 registers are defined as follows
;        Register       Reset State        Comments
;     *  ADCTL          00UU,UUUU          Reset below
;        ADR1-4         XXXX,XXXX          Read Only (Data)
;     *  BAUD           0000,0UUU          SCI Baud Reset below
;     *  BPROT          1110,0000          EEPROM protect reset below
;        CFORC          0000,0000          Timer compare force reg, not used
;     *  CONFIG         0000,0UUU          Implemented in EEPROM
;        COPRST         UUUU,UUUU          COP reset reg, not used
;     *  DDRC           0000,0000          Data dir C, reset below
;     *  DDRD           0000,0000          Data dir D, reset below
;        HPRIO          UUUU,0101          IRQ change. OC1 hi pri (set below)
;        INIT           0000,0000          RAM/IO mapping unchanged
;     *  OC1D           0000,0000          Out comp 1 data set below
;     *  OC1M           0000,0000          Out comp 1 mask set below
;     *  OPTION         0001,0000          Option reset below
;        PACNT          XXXX,XXXX          Pulse accum data not used
;     *  PACTL          0000,0000          Pulse accum reg reset below
;        PIOC           0000,0U11          Parallel i/o control not used
;        PORTA          Z000,0ZZZ          Port A Data
;        PORTB          0000,0000          Port B Data
;        PORTC          ZZZZ,ZZZZ          Port C Data Hi Z inputs
;        PORTCL         ????,????          Port C Latched Data not used
;        PORTD          01ZZ,ZZZZ          Port D Data
;        PORTE          ZZZZ,ZZZZ          Port E Data a/d inputs
;        PPROG          0000,0000          EEPROM Prog reg
;     *  SCCR1          UU00,0000          SCI ctrl reg 1 reset below
;     *  SCCR2          0000,0000          SCI ctrl reg 2 reset below
;        SCDR           XXXX,XXXX          SCI data reg
;        SCSR           1100,0000          SCI status reg
;     *  SPCR           0000,01UU          SPI control reg reset below
;        SPDR           XXXX,XXXX          SPI data reg
;        SPSR           0000,0000          SPI status reg
;        TCNT           0000,0000          Timer reg, 16 bit, 0000 on reset
;     *  TCTL1          0000,0000          Timer ctrl 1, set below
;        TCTL2          0000,0000          Timer ctrl 2, capture disabled
;        TEST1          0000,X000          Factory test, not used
;        TFLG1          0000,0000          Timer out comp/in capt IRQ status
;        TFLG2          0000,0000          Timer TOF, RTI, PW IRQ status
;        TIC1-3         0000,0000          Input capture regs, not used
;     *  TMSK1          0000,0000          Main timer IRQ msk, set below
;     *  TMSK2          0000,0000          Main timer IRQ msk, set below
;        TOC1-5         0000,0000          Timer Out comp regs, not used
;

;****************************************************************************
;  This is where the program starts in the EEPROM
;
Reset:          org     0f800h            ;start at top eeprom

;****************************************************************************
;  Register Initialization
;
                ldx     #REGBAS           ;offset to registers
                ldaa    #93h              ;Enable charge pump, A/D->E
                staa    OPTION,x          ;cop timeout 1.049 sec
                clra                      ;disable eeprom protection
                staa    BPROT,x           ;Clear the Block Protect Register
                staa    SCCR1,x           ;configure sci serial port

;  8 data bits 1 stop bit
;
                ldaa    #00001011b        ;Set OC1 priority IRQ
                staa    HPRIO,x           ;OC1 highest maskable pri

                ldd     #2000             ;Timer Functions
                std     TOC1,x            ;Set 1ms irq cycle for oc1
                                          ; 1msec/.5us/tick = 2000

                ldd     #20000            ;Set 10ms irq cycle for oc2
                std     TOC2,x            ; 10ms/.5us/tick =
 
                ldaa    #11111100b        ;Port C DDR
                staa    DDRC,x            ;C7-C2 Outputs, C1-C0 Inputs
 
                ldaa    #00111010b        ;Port D DDR
                staa    DDRD,x            ;D5,4,3,1 outputs 2,0 inputs
 
                ldaa    #10001000b        ;Pulse Accumulator Control Reg
                staa    PACTL,x           ;DA7, DA3 out, RTI IRQ e/2^16
 
                ldaa    #00001100b        ;enable sci serial port
                staa    SCCR2,x           ;rx, tx activated, no irq
 
                ldaa    #00110000b        ;set baud of 9600
                staa    BAUD,x            ;
 
                ldaa    #01010000b        ;spi control reg
                staa    SPCR,x            ;No IRQ, Sys Enble, clk E/2
 
;****************************************************************************
;  Variable Initialization
;  ram wipe : This is where we clear all RAM memory.
;
Ram_Clear:      ldx     #0                ;Get the starting address
                clra                      ;Clear the Accumulator
Ram_Wipe:       staa    0,x               ;Clear this address
                inx                       ;Increment our address offset index
                cpx     #0ffh             ;Have we cleared 255 bytes yet
                bls     Ram_Wipe          ;No, loop till we're done
 
                lds     #0ffh             ;stack at top of ram
                ldaa    #08h              ;Init b0 coefficient
                staa    vb                ;8 bit fraction: 0.03125
                staa    ib                ;8 bit fraction: 0.03125
                staa    t1b               ;8 bit fraction: 0.03125
                staa    t2b               ;8 bit fraction: 0.03125
 
                ldaa    #0f0h             ;Init a1 coefficient
                staa    va                ;8 bit fraction: 0.93750
                staa    ia                ;8 bit fraction: 0.93750
                staa    t1a               ;8 bit fraction: 0.93750
                staa    t2a               ;8 bit fraction: 0.93750
 
                ldaa    #244              ;Equates to 1 second
                staa    four_msec_count   ;Preset power on counter to 1 second
                staa    mfour_msec_count  ;Preset the mode timer to 1 second

;****************************************************************************
;  init_dlcs 
;  Description:  Initializes the DLCS.  DLCS is reset, a config/commandbyte
;                is sent to the DLCS and a short delay is implemented to
;                ensure latching of the configuration data.
;  Prog flow:    Linear execution, calls putdlcs.
;  Regs Altered: x, a, b
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;
Init_Dlcs:      ldx     #REGBAS           ;Offset to registers
                bclr    PORTA,x,RST       ;Assert DLCS *RST
                nop                       ;sleep for .5us
                nop                       ;sleep for .5us
                bset    PORTA,x,RST       ;DLCS *RST pin
                                          ;Negate DLCS *RST
                clra                      ;DLCS config: 0000,0000
                                          ;No tst, irq en, 00 osc, 0 4x
                ldab    #18h              ;Load Cmd config:  0001,1000
                                          ;Gen: NOP, Typ: #cnfg, Rx: NOP
                jsr     PutDlcs           ;Send message to DLCS

;****************************************************************************
;  Start the A/D conv.
;
                ldaa    #30               ; initiate the A/D
                staa    ADCTL,x           ; a/d conversion
 
;****************************************************************************
;  Interrupts Enabled
;
                ldaa   #01000000b         ;Timer Functions
                staa   TMSK2,x            ;IRQ on  RTI
                                          ; E/1 = .5us/cycle
 
                bset   TMSK1,x, 80h       ;enable oc1 irq (a/d)
 
                                          ; mode=0 (pud)
                cli                       ;enable rti irq
                                          ;system clock starts ticking
 
;****************************************************************************
;  A/D Settling Time  Delay
 
A2d_Wait:       jsr     Reset_Watchdog    ;give analog 1sec to settle
                ldaa    four_msec_count   ;See how long it's been
                cmpa    #1                ;Have we waited long enough
                bhi     A2d_Wait          ;No, keep waiting
 
;******************************************************************************
;  Read the ports, then set PA7 (Discrete Output).  X Register is set in the
;  Read_Ports routine so no need to set it here.
;
Set_Contact:    jsr     Read_Ports        ;Get state of Port A, and save it
                bset    PORTA,x,080h      ;Turn on PA7, Discrete Output

;****************************************************************************
;  Power Up Delay
;
                ldaa    #pud              ;Get the power up delay
Pud_Loop:       jsr     Reset_Watchdog    ;Go reset the watchdog
                cmpa    one_sec_count     ;Have we waited long enough yet?
                bgt     Pud_Loop          ;Loop till we time out

;****************************************************************************
;  More Interrupts Enabled
;
;  More_irq label added when this routine was moved from prior to the Power Up
;  Delay loop, to after the loop.
;
More_Irq:       bset    TMSK2,x, 80h      ;enable tof irq (Schedule J1850)
                bset    TMSK1,x, 40h      ;enable oc2 irq (check fault)

;****************************************************************************
;  Calculate State Of Charge at Start - 12 May 95 by J.W. Hagen, HTSC
;  Description:  Calculates the approximate state of charge for the battery
;                pack prior to entering the charging cycle.
;  Prog flow:    Linear execution
;  Regs Altered: D,X
;  Global Read:  v_at_start_up
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Calc_Soc_Start: ldab    v_raw             ;Get the raw voltage
                stab    v_at_start_up     ;Save as the starting voltage
                cmpb    #VMIN             ;Is it at least Vmin
                bhs     Calc_Soc_Star1    ;Yes, we're ok
                ldab    #VMIN             ;Get Vmin

Calc_Soc_Star1: subb    #ZERO_REF         ;Subtract our zero reference
                bcc     Calc_Soc_Star2    ;Not set if we didn't borrow
                negb                      ;Get the 2's compliment
                bset    flag1,04h         ;Set bit 2 to show as negative number

Calc_Soc_Star2: ldaa    #100              ;To give us an integer percentage
                mul                       ;Apply it
                ldx     #BATT_ENERGY      ;1.2 * #Batteries
                idiv                      ;Yields the startup State of Charge
                xgdx                      ;Move it to the A&B, use the LSByte
                brset   flag1,04h,Calc_Soc_Star3   ;Flag clear, press on
                cpd     #100              ;How does it compare to 100%
                bhs     Calc_Soc_Star4    ;More than or same as 100%
                bra     Calc_Soc_Star5    ;0-99%, looking good

Calc_Soc_Star3: negb                      ;So we can save it as a negative #
                stab    soc_at_start      ;Save the negative value.  Happens
                                          ;when the battery is really low.
                clrb                      ;For our state of charge indication
                bra     Calc_Soc_Done     ;Branch and save

Calc_Soc_Star4: ldab    #99               ;Hard load a 99% value
Calc_Soc_Star5: stab    soc_at_start      ;Save our starting value
Calc_Soc_Done:  stab    ch_state          ;Our % State of Charge is in the B

;****************************************************************************
;  Load parameter data into RAM
;
                jsr     Ram_Init          ;Initialize RAM with parameter data

;****************************************************************************
;
;  Set the required flags into Flag1
;
;  The following routine copies the temperature compensation flag.  If set to
;  1, the temperature compensation routine is active.  If set to 0, the routine
;  is inactive.  The flag becomes Bit 0 in the Flag1 byte.
;
                ldaa    #TEMP_COMP_FLAG   ;Get the Temperature Comp Flag

;  Save the new set of flags for Flag 1.
;
                staa    flag1     ;Save as the new value

;****************************************************************************


;****************************************************************************
;
;  Set the required flags into Flag2
;
;  The following routine copies the Auxiliary Battery monitor flag into Special
;  Flag 2, Bit 3.  If set to 1, the Auxiliary Battery voltage is monitored for
;  upper and lower limits.  Should the voltage go higher or lower than the pre-
;  defined limits, Fault Mode is entered.
;
                ldaa    #MON_AUX_BATT     ;Move  Monitor Aux Battery to Bit 0.

;  The following routine copies the Dolphin monitor flag into Special Flag 2,
;  Bit 2.  If set to 1, the Dolphin unit is monitored for activity.  If the lid
;  is opened or the Fet turns off, Fault Mode is entered.
;
                lsla                      ;Move MON_AUX_BATT to Bit 1.
                oraa    #MON_DOLPHIN      ;Move MON_DOLPHIN to Bit 0

;  The following routine copies the Temperature Sensor monitoring flag into
;  Flag 2, Bit 1.  If set to 1, the DTemperature Sensors are monitored for
;  over and under temperature limits.  If any of the limits defined in the
;  customer.h file are exceeded, Fault Mode is entered.
;
                lsla                      ;Move MON_AUX_BATT to Bit 2.
                                          ;Move MON_DOLPHIN to Bit 1
                oraa    #MON_TEMP_SENSE   ;Move MON_TEMP_SENSE to Bit 0

                lsla                      ;Move MON_AUX_BATT to Bit 3.
                                          ;Move MON_DOLPHIN to Bit 2
                                          ;Move MON_TEMP_SENSE to Bit 1
                                          ;Move a 0 into Bit 0.

;  Save the new set of flags for Special Flag 2.
;
                staa    flag2     ;Save as the new value

;****************************************************************************
;
;  Set the required flags into Flag3
;
;  The following routine copies the Bypass Trip Point flags into Special
;  Flag 3.  If set to a 1, the Mode trip points are ignored and mode duration
;  is controlled by the amount of time set in the appropriate Tn_HOURS and
;  Tn_MINS.
;
                ldaa    #BYPASS_TRIPPT1   ;Move Bypass_TripPt1 to Bit 0.

                lsla                      ;Move Bypass_TripPt1 to Bit 1.
                oraa    #BYPASS_TRIPPT2   ;Move Bypass_TripPt2 to Bit 0.

                lsla                      ;Move Bypass_TripPt1 to Bit 2.
                                          ;Move Bypass_TripPt2 to Bit 1.
                oraa    #BYPASS_TRIPPT3   ;Move Bypass_TripPt3 to Bit 0.

                lsla                      ;Move Bypass_TripPt1 to Bit 3.
                                          ;Move Bypass_TripPt2 to Bit 2.
                                          ;Move Bypass_TripPt3 to Bit 1.
                oraa    #BYPASS_TRIPPT4   ;Move Bypass_TripPt4 to Bit 0.

                lsla                      ;Move Bypass_TripPts 1 more bit
                lsla                      ;Move Bypass_TripPts 1 more bit
                lsla                      ;Move Bypass_TripPts 1 more bit
                lsla                      ;Move Bypass_TripPts 1 more bit

;  Save the new set of flags for Special Flag 3.
;
                staa    flag3     ;Save as the new value

;****************************************************************************
;
;  Set the required flags into Flag4
;
;  The following routine copies the DO_TOP_OFFS flag into Special Flag 4, Bit
;  3.  If set to 1, Battery Pack topping off will occur at the time period
;  specified.
;
                ldaa    #DO_TOP_OFFS      ;Move Leveling Flag to Bit 0
                lsla                      ;Move Leveling Flag to Bit 1.
                lsla                      ;Move Leveling Flag to Bit 2.
                lsla                      ;Move Leveling Flag to Bit 3.

;  Save the new set of flags for Special Flag 4
;
                staa    flag4     ;Save as the new value

;****************************************************************************
;  Sample_periods is used in the CTRL: module to limit the update rate to
;  the charge command.  Preset the counter to a value to force it into an
;  immediate update at startup.
;
                ldaa    #200              ;Set to a high value ( > 100 )
                staa    sample_periods    ;Control Loop sample periods complete

;  This flag is used to determine a fast or slow iteration on the error
;  generated by SetPoint - Measured in the CTRL: module.  Set the flag to start
;  a fast iteration.
;
                bset    flag3,08h         ;For positive error calculations

;*****************************************************************************
;  Perform last minute changes prior to entering the main program loop.
;
                clr     brownout_secs     ;Reset the counter
                clr     brownout_wait     ;Reset the counter

                inc     mode              ;Increment to Mode 1
                bset    flag1,010h        ;Show the transition to mode 1
                bset    flag3,01h         ;Set Bit 0 as Charge Port 1 CP

;*****************************************************************************
;  Main program
;  Description:  Executes all non-IRQ system and diagnostic functions.
;  Prog flow:    Continuous loop.  All system inputs are handled on interrupt.
;  Regs Altered: A,X   
;  Global Read:  i_raw
;  Global Write: None.
;  return value: None.
;
Main:           jsr     Temp_Comp         ;Perform temperature compensation
                jsr     Calc_Soc          ;Calculate the State Of Charge
                jsr     Ram_Out           ;Send RAM data out the RS232 port
                jsr     Clear_PA7         ;Determine status for discrete output
                jsr     Read_Ports        ;Update Port information
                bsr     Leveling          ;See if leveling is needed
                bra     main              ;Loop till we quit 

;******************************************************************************


;******************************************************************************
;  Leveling       02 Apr 96   John W. Hagen III, HTSC
;  Description    This module monitors the traction pack and the auxiliary
;                 battery, after the charge controller has entered completion
;                 mode.  Monitoring will occur only if the DO_TOP_OFFS flag in
;                 the customer.h file is set to a one (1).
;
;                 The auxiliary battery voltage is monitored at approximately
;                 10 hz.  Should the voltage fall below V_AUX_BATT_MON
;                 (defaulted to approximately 12.4vdc), the Dolphin system
;                 will be enabled (PA7 turned on), a 5 second delay will be
;                 entered, charging will start in MODE 4, lasting for the
;                 duration specified in MODE 4's parameters.  Upon completion
;                 of the time specified, charging will terminate, and
;                 completion mode will be re-entered.  Auxiliary battery
;                 voltage monitoring will then occur as before.
;
;                 The traction pack voltage will be monitored once per hour.
;                 Should the voltage fall below V_START_LEVEL as defined in the
;                 customer.h file, the Dolphin system will be enabled (PA7
;                 turned on), a 5 second delay will be entered, charging will
;                 start in MODE 2, utilizing normal charge algorithms, trip,
;                 and set points. Upon completion, charging will terminate,
;                 and completion mode will be re-entered.  Traction pack
;                 voltage monitoring will then occur as before, once per hour.
;
;  Program Flow   Non-Interrupt.  Runs approximately 10 hz in Completion Mode.
;  Regs Altered:  a, x, y
;  Global Read:   
;  Global Write:  
;  Return value:  None.
;
;  See if the present mode is completion mode.  If not, exit.
;
Leveling:       ldaa    mode              ;Find out the present mode
                cmpa    #MODE254          ;Is it completion mode
                bne     Leveling_Rts      ;No, exit
                clr     brownout_secs     ;Clear the brownout counter

;  If the leveling flag is set (=1), topping off is desired.  If not set (=0),
;  exit.
;
                brclr   flag4,08h,Leveling_Rts  ;If clear, exit

;  Check to see if the aux battery needed to be charged, as determined by a
;  previous entry.  If so, bypass checking the battery again.
;
                brset   flag4,020h,Level_Ck_PA7 ;If clear, go check the time

;  Test the aux battery voltage.  If at 12.4 volts or less, enter Mode 4 and
;  begin charging, to bring the aux battery voltage back up to where it should
;  be.
;
Level_Aux_Batt: ldaa    v_auxbattery      ;Get the aux battery voltage
                cmpa    #V_AUX_BATT_MON   ;Has it reached the recharge point
                bhi     Level_Ck_Time     ;No, check the present time
                bset    flag4,020h        ;Show the aux battery needs a charge
                bra     Level_Ck_PA7      ;Needs a charge, setup PA7 to begin

;  In Completion Mode.  Next see if just starting a new hour.
;
Level_Ck_Time:  tst     mone_min_count    ;Are we at Zero minutes
                bne     Leveling_Clr      ;No, then it's not time.  Clear flag.

;  Now test to see if we have already sampled for this hour.  If so, exit and
;  wait until the start of a new hour (minutes = 0).
;
                brset   flag4,02h,Leveling_Rts  ;If set, don't process

;  If this is the first time through, enable the PA7 relay to turn on the
;  Dolphin unit.  This allows voltage to become available for measurement.
;
Level_Ck_PA7:   brset   flag4,04h,Level_PA7_On  ;If set, check how long

;  PA7 was not previously enabled.  Turn it on and exit.
;
                bset    flag4,04h         ;Show PA7 as enabled
                ldx     #REGBAS           ;Base Register offset
                bset    PORTA,X,080h      ;Enable PA7 to start the Dolphin
                bra     Leveling_Rts      ;Exit for 5 seconds

;  The Dolphin Unit has been previously turned on.  If it's been on 5 seconds
;  then proceed with testing the battery pack voltage.  If less than 5 seconds
;  then exit.
;
Level_PA7_On:   ldaa    mone_sec_count    ;How many seconds has it been
                cmpa    #5                ;Has it been at least 5 seconds
                bcs     Leveling_Rts      ;No, exit

;  Check to see if the aux battery needed to be charged.  If so, load the
;  parameters and start charging.
;
                brclr   flag4,020h,Level_Ck_Pack   ;If clear, go test batt pack

;  Aux battery has fallen to 12.4 volts or less.  Change to Mode 4, set any
;  required flags and begin charging.
;
                ldaa    #MODE4            ;Get the value for Mode 4
                staa    mode              ;Now in Mode 4
                bset    flag4,010h        ;Show aux battery is charging
                inc     aux_batt_entry    ;Show another charging cycle
                bra     Level_SetUp       ;Go finish setting up leveling mode

;  In completion mode, Dolphin has been on for at least 5 seconds.  See what
;  the battery pack raw voltage looks like.
;
Level_Ck_Pack:  bset    flag4,02h         ;Indicate Voltage Sample taken
                ldaa    v_raw             ;Get the battery pack raw voltage
                cmpa    #V_START_LEVEL    ;Has it fallen to leveling voltage
                bhi     Leveling_Rts      ;No, it's higher, exit

;  Completion Mode, battery pack raw voltage is at or below the preset voltage
;  in which leveling should start.  Set up the necessary flags and begin
;  charging in Mode 2.
;
                bset    flag4,01h         ;Indicate leveling has been entered.
                inc     level_entry       ;Show another entry into leveling

;  Show Mode 2 as present charging mode.
;
                ldaa    #MODE2            ;Load Mode 2
                staa    mode              ;Back into Mode 2

;  Finish setting up to restart the charging process in Mode 4.
;
Level_SetUp:    ldaa    #080h             ;Set the leveling bit
                staa    ch_state          ;Show the charger in leveling
                bset    flag1,010h        ;Show Mode transition
                clr     ch_status         ;Show charging in progress.
                rts                       ;Back to the main loop

Leveling_Clr:   bclr    flag4,02h         ;Show the voltage check available
Leveling_Rts:   rts                       ;Back to the main loop

;******************************************************************************


;******************************************************************************
;  Tof_Irq
;  Description:  Timer overflow IRQ routine.  Sets a flag for diagnostics &
;                schedules the J1850 messages to go out.  Power request
;                message transmits every 98ms, state of charge j1850 message
;                transmits every  1.048sec, charge status j1850 message
;                transmits only when in modes 5 or 6 every 131ms.
;
;                Modified tof_irq2 to send ch_command every 98msec.
;                Modified tof_next to process CONTROL every 1048msec.
;
;                2/01/95 JWH
;
;  Prog flow:    IRQ service.  Scheduled to happen every 32.77ms.
;  Regs Altered: x, a.
;  Global Read:  tof_cnt
;  Global Write: None.
;  Return value: None.
;
TOF_CLEAR:      equ     7fh               ;bclr to TFLG2 to clear IRQ
 
;  No J1850 messages are scheduled to occur if in Mode 0.
;
Tof_Irq:        tst     mode              ;if mode=0, then no J1850 messages
                beq     Tof_End           ;This is Mode 0, exit
 
;  Increment our Tof_Irq execution counter.
;
                inc     tof_cnt           ;Keep count of TOF IRQs

                ldaa    tof_cnt           ;Execution cycle counter
                anda    #5h               ;mask, mod 32 counter 01Feb95
                                          ;when using mod 32, control
                                          ;routine happens every
                                          ;32*32.77ms=1.04sec
                bne     Tof_Irq1          ;Time to cogitate?
                jsr     Control           ;Review data, process outputs
                bra     Tof_End           ;Exit
 
;  Determine the present Mode.  If Modes 1-4, send Ch_Command or Ch_State msgs
;  as scheduled.  If Mode 254 or 255, send Charge Status Message only.
;
Tof_Irq1:       ldaa    mode              ;Get the current mode
                cmpa    #MODE254          ;Are we in Completion or Fault Mode
                bcs     Tof_Irq2          ;Not Completion or Fault Mode
                clr     mode_delay        ;Clear the Mode Delay seconds counter

;  Schedules Charge Status Message to J1850 bus, when in Modes 254 or 255 only.
;  The value used to AND with tof_cnt determines the rate in which the Charge
;  Status messages are sent to the J1850 bus.  Currently set for 03h.  This
;  provides an execution rate of 7.63hz = (32.77ms * 4) = every 131msec.
;
                ldaa    tof_cnt           ;Load the execution cycle counter
                anda    #3h               ;Send Ch_Status every 4 * 32.77ms
                cmpa    #1h               ;Time to send ch_status?
                bne     Tof_End           ;Not time to send message, press on

;  Send the Charge Status Message and exit.
;
                bsr     Send_Msg2         ;Send Charge Status Message
                bra     Tof_End           ;Exit

 
;  Schedules Charge Command Message to J1850 bus, Modes 1 thru 4 only.
;  The value used to AND with tof_cnt determines the rate in which the Charge
;  Command messages are sent to the J1850 bus.  Currently set for 03h.  This
;  provides an execution rate of 7.63hz = (32.77ms * 4) = every 131msec.
;
Tof_Irq2:       ldaa    tof_cnt           ;Load the execution cycle counter
                anda    #3h               ;Send Ch_Command every 4 * 32.77ms
                cmpa    #1h               ;Time to send ch_command?
                bne     Tof_Irq3          ;Not time yet, check Ch_State Message

;  Send the Charge Command Message and exit.
;
                bsr     Send_Msg1         ;Send Charge Command Message
                bra     Tof_End           ;Exit
 
;  Schedules Charge State Message to J1850 bus, Modes 1 thru 4 only.
;  The value used to AND with tof_cnt determines the rate in which the Charge
;  State messages are sent to the J1850 bus.  Currently set for 01h.  This
;  provides an execution rate of .954hz = (32.77ms * 32) = every 1.048sec.
;
Tof_Irq3:       ldaa    tof_cnt           ;Load the execution cycle counter
                anda    #1fh              ;Send Ch_State every 32 * 32.77ms
                cmpa    #1fh              ;Time to send ch_state?
                bne     Tof_End           ;Not time yet, exit

;  Send the Charge State Message and exit.
;
                bsr     Send_Msg3         ;Send Charge State Message

;  This routine sets us up to exit the interrupt.
;
Tof_End:        ldx     #REGBAS           ;Offset to registers
                bclr    TFLG2,x,TOF_CLEAR ;Clear the interrupt
                rti                       ;Return from the interrupt
 
;****************************************************************************


;****************************************************************************
;  send_msg_x (x = 1, 2, 3)
;  Description:  Sends one of three J1850 charging messages.
;  1. Charge Command Message: Subaddress = 0x10, Data: = see ch_command      
;  2. Charge Status Message:  Subaddress = 0x11, Data: = see ch_status
;  3. State of Chrge Message: Subaddress = 0x12, Data: = see ch_state
;  Prog flow:    Linear, called by tof_irq.
;  Regs Altered: a, b, x
;  Global Read:  ch_command, ch_status, ch_state
;  Global Write: None.
;  Return value: None.
;
                                          ;Message #10
c_cmd:          dfb     20h,04h           ;CMD:0000,0100 DAT:0001,0000
                                          ; cmd-a tx byte, dat:address

                                          ;Message #11
c_sts:          dfb     21h,04h           ;CMD:0000,0100 DAT:0001,0001
                                          ; cmd-a tx byte, dat:address

                                          ;Message #12
c_ste:          dfb     22h,04h           ;CMD:0000,0100 DAT:0001,0001
                                          ; cmd-a tx byte, dat:address
 
send_msg1:      ldx     c_cmd             ;Get the adrs info
                ldaa    ch_command        ;Load THE DATA
                bra     send_a_message    ;Send the message
 
send_msg2:      ldx     c_sts             ;Get the adrs info
                ldaa    ch_status         ;Load THE DATA
                bra     send_a_message    ;Send the message
 
send_msg3:      ldx     c_ste             ;Get the adrs info
                ldaa    ch_state          ;Load THE DATA
 
send_a_message: psha                      ;Store data
                bsr     tx_the_header     ;Send the sys header
                xgdx                      ;Get data/cmd for address
                bsr     putdlcs           ;Send the adrs info
                pula                      ;Get data
                ldab    #0ch              ;Load cmd for last data
                bsr     putdlcs           ;Send the cmd/data pair
                rts

;****************************************************************************

 
;****************************************************************************
;  tx_the_header
;  Description:  Loads and sends the standard system header to J1850
;  Prog flow:    Linear, called by send_msgx.
;  Regs Altered: a, b
;  Global Read:  None.
;  Global Write: None.
;  Return value: None.
;
 
tx_the_header:  ldd     #8814h            ;Load data/cmd
                                          ;cmd=14=load as 1st byte of message
                bsr     putdlcs           ;Send the data/cmd pair
 
                ldd     #0f204h           ;Load data/cmd
                                          ;cmd=04=load as trasmit data
                bsr     putdlcs           ;Send the data/cmd pair
 
                ldd     #7204h            ;Load data/cmd
                                          ;cmd=04=load as trasmit data
                bsr     putdlcs           ;Send the data/cmd pair
                rts                       ;Go away

;****************************************************************************


;****************************************************************************
;  putdlcs
;  Description:  Sends command/data seq to the DLCS chip.  Checks for J1850
;                errors in the status byte (bus short).
;  Prog flow:    Linear execution, waits for data transfer complete after
;                each byte sent (data and command).
;  Regs Altered: a, b
;  Global Read:  None.
;  Global Write: bad_status.
;  Passed value: data in reg a, command in reg b.
;  Return value: data in reg b, status in reg a.
;
CS:             equ     010h              ;DLCS *CS bit mask
 
putdlcs:        pshx                      ;Save x
                ldx     #REGBAS           ;Offset to registers
                bclr    PORTA,x,CS        ;Assert DLCS *CS
                staa    SPDR,x            ;Send data byte
                clra                      ;Get set for compare
putdlcs1:       cmpa    SPSR,x            ;SPI transfer ok check
                bpl     putdlcs1          ;Loop
                brclr   SPDR,x,08h,putdlcs2 ;Check status for short
                ldaa    SPDR,x            ;Retrieve status
;               staa    bad_status        ;Store bad status
putdlcs2:       ldaa    SPDR,x            ;Retrieve status
                stab    SPDR,x            ;Send commandbyte
                clrb                      ;Get set for compare
putdlcs3:       cmpb    SPSR,x            ;SPI transfer ok check
                bpl     putdlcs3          ;Loop
                ldab    SPDR,x            ;Retrieve recieve data
                bset    PORTA,x,CS        ;Negate DLCS *CS
                pulx                      ;Restore x
                rts

;****************************************************************************


;****************************************************************************
;  rti_irq       Modified 12 Dec 94 by JWH for BrownOut
;
;  Description:  Real time IRQ routine. Updates system clocks
;  Prog flow:    IRQ service.  Scheduled to happen every 4.10ms.
;  Regs Altered: x, a, b
;  Global Read:
;  Global Write: None.
;  Return value: None.
;
RTI_CLEAR:      equ     40h               ;Write to TFLG2 to clear IRQ
;
; Four_msec_count pre-loaded with 244, which is 1 seconds worth
; One_sec_count cleared to zero.
;
Rti_Irq:        brclr   Flag1,010h,Rti_Irq1 ;Set indicates mode changed
                bclr    Flag1,010h        ;Time to clear it out

;  Since we changed modes, we need to reset our mode clock timers.
; 
Rti_Irq0:       clr     mone_hour_count   ;reset mode clock
                clr     mone_min_count    ;reset mode clock
                clr     Mode_Delay        ;reset mode delay clock
                clr     mone_sec_count    ;Clear the seconds counter
                ldaa    #244              ;1 seconds worth of data
                staa    mfour_msec_count  ;reset mode clock

;  This routine tracks the time elapsed since power-up
;  
Rti_Irq1:       dec     four_msec_count   ;decrement our msecond timer
                tst     four_msec_count   ;Have we reached zero yet
                bne     rti_irq2          ;No, keep working
                ldaa    #244              ;We completed 1 second, load it again
                staa    four_msec_count   ;re-load 4.1ms counter with 1 second
                inc     BrownOut_Secs     ;Increment BrownOut Seconds Counter
                inc     BrownOut_Wait     ;Increment BrownOut Recovery Counter
                inc     one_sec_count     ;Increment 1 second from the count
                inc     Mode_Delay        ;Increment 1 sec delay for Mode 2&4
                ldaa    one_sec_count     ;Get the current seconds counter
                cmpa    #60               ;Have we reached 60 seconds yet
                blt     rti_irq2          ;No, keep working
                clr     one_sec_count     ;Clear the seconds counter
                inc     one_min_count     ;Increment the minute counter
                ldaa    one_min_count     ;Let's see what it has
                cmpa    #60               ;Have we reached an hour yet
                blt     rti_irq2          ;Less than an hour, keep working
                clr     one_min_count     ;Clear the minute counter
                inc     one_hour_count    ;Increment the hour counter
 
;  This routine tracks the time elapsed in current mode
;
rti_irq2:       dec     mfour_msec_count  ;Decrement the mode msecond counter
                tst     mfour_msec_count  ;Have we completed 1 second yet
                bne     rti_end           ;No, bail out
                ldaa    #244              ;Yes, load equal to 1 second
                staa    mfour_msec_count  ;re-load 4.1ms counter with 1 second
                inc     mone_sec_count    ;Increment the seconds counter
                ldaa    mone_sec_count    ;Get the current mode seconds counter
                cmpa    #60               ;Have we reached 60 seconds yet
                blt     rti_end           ;No, bail out
                clr     mone_sec_count    ;Clear the mode seconds counter
                inc     mone_min_count    ;Increment the minute counter
                ldaa    mone_min_count    ;Let's see what's there
                suba    #60               ;Have we completed an hour yet
                blt     rti_end           ;No, press on
                clr     mone_min_count    ;Clear the mode minute counter
                inc     mone_hour_count   ;Increment the mode hour counter

;  This routine sets us up to exit the interrupt.
;
rti_end:        ldx     #REGBAS           ;Load the base address
                ldaa    #RTI_CLEAR        ;Load RTI clear mask
                staa    TFLG2,x           ;Clear RTI
                rti                       ;Return from the interrupt
 
;****************************************************************************

 
;****************************************************************************
;  oc1_irq
;  Description:   Retrieves v, i, temp a/d values and calls iir filter routine
;                 scheduled to happen every 1msec
;
;                  Modified to select Group 1 (Channels PE0-PE3) on the odd
;                 number pass; set a flag, then select Group 2 (Channels
;                 PE4-PE7) on the even number pass.  In reality, each A/D is
;                 being read every 2msec.  11 Mar 95  JWH
;
;                 Modified to add PE6 and PE7 as thermal management inputs for
;                 the 5.0 hardware.  15 Aug 95  JWH
;
;  Prog flow:     IRQ service.  Calls iir_v iir_t
;  Regs Altered:  x, a, b.
;  Global Read:   v_raw, i_raw, t1_raw, t2_raw
;  Global Write:  v_raw, v_filt, t1_raw, t1_filt, t2_raw, t2_filt
;  Return value:  None.
;
OC1_CLEAR:      equ     07fh              ;bclr to TFLG1 to clear IRQ

Oc1_Irq:        ldx     #REGBAS           ;Offset to registers
                ldd     TOC1,x            ;Get old compare value
                addd    #2000             ;Set 1ms irq cycle for oc1
                std     TOC1,x            ;Set 1ms irq cycle for oc1
                                          ; 1msec/.5us/tick = 2000

Oc1_Group_1:    brset   Flag2,80h,Oc1_Group_2 ;Flag set, do next group

;  Read analog channels PE0 - PE3, then set a flag indicating these channels
;  have been read.
;
Read_V_Raw:     ldaa    ADR1,x            ;get v a/d reading
                staa    v_raw             ;store to raw loc
                staa    vx                ;store to filter input
                ldx     #vx               ;regX=address of vx
                bsr     iir               ;do iir
                std     v_filt            ;Store result

Read_I_Raw:     ldaa    ADR2,x            ;get i a/d reading
                staa    i_raw             ;store to raw loc
                staa    ix                ;store to filter input
                ldx     #ix               ;regX=address of vx
                bsr     iir               ;do iir
                std     i_filt            ;Store result

Read_T1_Raw:    ldaa    ADR3,x            ;read temp1
                staa    t1_raw            ;save temp1 reading
                staa    t1x               ;store to filter input
                ldx     #t1x              ;regX=address of t1x
                bsr     iir               ;do iir
                std     t1_filt           ;Store result
 
 
Read_T2_Raw:    ldaa    ADR4,x            ;read temp2
                staa    t2_raw            ;save temp2 reading
                staa    t2x               ;store to filter input
                ldx     #t2x              ;regX=address of t2x
                bsr     iir               ;do iir
                std     t2_filt           ;Store result

                ldx     #REGBAS           ;offset to registers
                ldaa    #34h              ;Set for ch 4-7 a/d once
                staa    ADCTL,x           ;Start. Use 'em next time

                bset    Flag2,80h         ;Set to indicate 1st group is done
                bra     Oc1_Irq_Exit      ;Finished with Group 1, exit

;  Read analog channels PE4 - PE5, then clear a flag indicating these channels
;  have been read.
;
Oc1_Group_2:    ldaa    ADR1,x            ;Get v a/d reading
                staa    v_auxbattery      ;Store to raw auxiliary battery

                ldab    ADR2,x            ;Get v a/d reading
                stab    v_precharge       ;Store to raw precharge

                ldab    ADR3,x            ;Get v a/d reading
                stab    v_conv_box        ;Conversion Box Temperature

                ldab    ADR4,x            ;Get v a/d reading
                stab    v_ch_port         ;Charge Port Temperature

                ldaa    #10h              ;Set for ch 0-3 a/d once
                staa    ADCTL,x           ;Start. Use 'em next time
                bclr    Flag2,80h         ;Clear to indicate 2nd group is done

;  This routine sets us up to exit the interrupt.
;
Oc1_Irq_Exit:   ldx     #REGBAS           ;Offset to registers
                bclr    TFLG1,x,OC1_CLEAR ;Clear IRQ
                rti                       ;Return from the interrupt

;****************************************************************************
 

;****************************************************************************
;  iir
;  Description:  Infinite Impulse Response filter for temp & voltage
;                measurements.
;  Prog flow:    Called from oc1_irq, linear execution.
;  Regs Altered: x, a, b.
;  Global Read:  16 bit result in location whose address is regX + 0
;  Global Write: None.
;  Return value: 16 bit result in regD
;
B:              equ     0
BX:             equ     2
W:              equ     4
A:              equ     7
Y:              equ     8

iir:            ldd     B,x               ;Get x and b in d
                mul                       ;x * b -> d
                std     BX,x              ;x * b -> bx
                addd    W,x               ;x * b + high 16 bytes of w
                std     Y,x               ;Store result
                clr     W,x               ;Clear high byte of w
                ldaa    A,x               ;Get a coefficient
                ldab    Y+1,x             ;Get low byte of y
                mul                       ;a * y lo -> d
                std     W+1,x             ;a * y lo -> 1st 16 lsbs of w
                ldd     A,x               ;Load a and y in d
                mul                       ;a * y hi -> d
                addd    W,x               ;a * y hi -> 1st 16 msbs of w
                addd    BX,x              ;x * b + a * y  -> d
                std     W,x               ;x * b + a * y  -> d
                ldd     Y,x
                ldx     #REGBAS           ;restore register offset to regX
                rts                       ;done
 
;****************************************************************************


;******************************************************************************
;  oc2_irq       Modified 13 Sep 95  JWH
;
;  Description:  Check for system faults (Mode 255) by looking for any or all
;                of the charge status conditions listed below.  Do not process
;                this routine if already in Completion Mode or Fault Mode.
;  Prog flow:    IRQ service
;  Regs Altered: y, a, b, x
;  Global Read:  
;  Global Write: ch_status.
;  Passed value: None.
;  Return value: Status in reg A.
;
;  NOTE:  In the following bit patterns, two bits have specific meaning.  They
;         are as follows:
;
;           Bit 0 indicates Charge Complete.
;           Bit 1 indicates Charging Failure
;
;     01h   =  Charge is Complete                                 0000 0001
;     06h   =  V_Filt  < VMIN                                     0000 0110
;     0Ah   =  T1_Filt > TempMax                                  0000 1010
;     0Eh   =  Contactor (Discrete PA2) Failed To Engage/Hold     0000 1110

;     12h   =  T2_Filt > TempMax                                  0001 0010
;     16h   =  SCM J1850 Transmitter Failed (BrownOut)            0001 0110
;     1Ah   =  Auxiliary Battery Under Voltage                    0001 1010
;     1Eh   =  Auxiliary Battery Over Voltage                     0001 1110

;     22h   =  One_Hour_Count > TMax                              0010 0010
;     26h   =  Dolphin Precharge FET is OFF                       0010 0110
;     2Ah   =  Dolphin Lid Switch if Open                         0010 1010
;     2Eh   =  Dolphin Fet Error, Unknown as to why.              0010 1110

;     42h   =  Temp Discrete Failure, Port A                      0100 0010
;     46h   =  Tot_Amp_Mins > Max_Amp_Mins (Exceeded Amphr Limit) 0100 0110
;     4Ah   =  Charging Time Limit Exceeded - Hours               0100 1010
;     4Eh   =  Charging Time Limit Exceeded - Hours & Minutes     0100 1110

;     82h   =  T1_Filt < TempMin                                  1000 0010
;     86h   =  V_Filt > VMAX                                      1000 0110
;     8Ah   =  T2_Filt < TempMin                                  1000 1010
;     8Eh   =  Conversion Box Overtemp                            1000 1110

;     92h   =  Conversion Box Thermal Switch is Open              1001 0010
;     96h   =  Charge Port RTD Device Shorted                     1001 0110
;     9Ah   =  Charge Port Thermistor Open                        1001 1010
;     9Eh   =  Spare                                              1001 1110

;     A2h   =  Spare                                              1010 0010
;     A6h   =  Spare                                              1010 0110
;     AAh   =  Spare                                              1010 1010
;     AEh   =  Spare                                              1010 1110

;     B2h   =  Spare                                              1011 0010
;     B6h   =  Spare                                              1011 0110
;     BAh   =  Spare                                              1011 1010
;     BEh   =  Spare                                              1011 1110

;     C2h   =  Temp Discrete Failure, Port C                      1100 0010
;     C6h   =  Spare                                              1100 0110
;     CAh   =  Spare                                              1100 1010
;     CEh   =  Spare                                              1100 1110

;     D2h   =  Badicheq Communications Lost                       1101 0010
;     D6h   =  Badicheq Sending Incorrect Frequency Pulses        1101 0110
;     DAh   =  Badicheq Charge Command > 255 exceeded 5 times     1101 1010
;     DEh   =  Spare                                              1101 1110


OC2_CLEAR:      equ     0bfh              ;bclr to TFLG1 to clear IRQ

;  Find out the present Mode.  If Completion or Fault Mode, don't process any
;  thing else here.  If in Fault Mode, one error was already found.  If 
;  Completion Mode, don't need to watch for errors since charging is complete.
;
Oc2_Irq:        ldaa    mode              ;What is the present mode
                cmpa    #MODE254          ;Is it Completion or Fault Mode
                bcs     Chk_AuxBattLo     ;No, process safety routine
                jmp     Chk_Fault_End     ;Finished here

;  Check Auxiliary Battery for Under Voltage.  This error will occur if the
;  V_AuxBattery is less than 2.45vdc.  Error detection is bypassed if Special
;  Flag 2, Bit 3 is cleared (MON_AUX_BATT).
;
Chk_AuxBattLo:  brclr   Flag2,08h,Chk_Dolphin  ;Flag off, skip check
                ldaa    V_AuxBattery      ;Get the present Aux Battery Voltage
                cmpa    #V_AUX_BATT_MIN   ;2.45v on A/D PE4 (11.0vdc AuxBatt)
                bhs     Chk_AuxBattHi     ;Looks okay, check for Over Voltage
                bset    Ch_Status, 1ah    ;update status word
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check Auxiliary Battery for Over Voltage.  This error will occur if the
;  V_AuxBattery is more than 3.19vdc.
;
Chk_AuxBattHi:  cmpa    #V_AUX_BATT_MAX   ;3.19vdc on A/D PE4
                blo     Chk_Dolphin       ;Looks okay, continue fault detection
                bset    Ch_Status, 1eh    ;update status word
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check the Dolphin unit for the following faults:  Lid Open or Fet Off.  If
;  the Fet is OFF, we either have the Dolphin contactor failure, Vsense >
;  Dolphin Vmax, or vehicle key was turned on.  Error detection is bypassed if
;  Special Flag 2, Bit 2 is cleared (MON_DOLPHIN).
;
Chk_Dolphin:    brclr   Flag2,04h,Chk_Vmax  ;Flag off, skip check
                ldaa    V_PreCharge       ;Get the A/D PE5 (Dolphin Fet status)
                cmpa    #V_DOLPHINNORM    ;If > V_DOLPHINNORM, Dolphin is dead
                blo     Chk_Vmax          ;Looks okay, continue fault detection

Chk_DolphinLid: cmpa    #V_DOLPHINLIDHI   ;If < V_DOLPHINLIDHI, Dolphin lid open
                bhs     Chk_DolphinFet    ;Dolphin Lid Closed, Check the Fet
                bset    Ch_Status, 2ah    ;Dolphin Lid is Open
                bra     Set_Fault_Mode    ;We're now in fault mode

Chk_DolphinFet: cmpa    #V_DOLPHINFETHI   ;If < V_DOLPHINFETHI, Dolphin Fet Off
                bhs     Chk_DolphinBad    ;Unknown Dolphin Error
                bset    Ch_Status, 26h    ;Dolphin Fet is Off
                bra     Set_Fault_Mode    ;We're now in fault mode

Chk_DolphinBad: bset    Ch_Status, 2eh    ;Dolphin Error Unknown
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check vehicle Voltage Sense (Vsense) against Voltage Maximum to see if we're
;  doing okay.  If not, set up as a fault.  Modified 07Jun94  JWH
;
;  check for Vsense > Vmax
;
Chk_Vmax:       ldaa    v_filt            ;Get the filtered voltage
                cmpa    #VMAX             ;Is it too much
                bls     Chk_Vmin          ;No, looks good, check Vmin
                bset    Ch_Status, 86h    ;update status word
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check for Voltage Sense (Vsense) against Voltage Minimum to see if we're
;  doing okay.  If not, set up as a fault.
;
;  check for Vsense < Vmin fault
;
Chk_Vmin:       ldaa    v_filt            ;Get the filtered voltage
                cmpa    #VMIN             ;Are we below minimums
                bhs     Chk_Temp_Sense    ;No, let's check the temperature
                bset    Ch_Status, 06h    ;update status word
                bra     Set_Fault_Mode    ;We're now in fault mode

;  Check to see if temperature sensor monitoring is enabled.  If not, bypass
;  the checks.
;
Chk_Temp_Sense: brclr   Flag2,02h,Chk_ToFault   ;Flag off, skip checks

;  Check for Temperature sense 1 (Tsens1) against Temperature Maximum to see
;  if we've exceeded TEMPMAX.  If so, set up as a fault.
;
;  Tsens1 > TEMPMAX is our comparison
;
Chk_T1_Max:     ldaa    t1_filt           ;Get the filtered temperature
                cmpa    #TEMPMAX          ;Have we reached the max temp
                bcs     Chk_T1_Min        ;No, look at the min
                bset    ch_status, 0ah    ;Update status word
Set_Fault_Mode: bra     Set_FaultMode     ;Now in fault mode

;  Check for Temperature sense 1 (Tsens1) against Temperature Minimum to see
;  if we've fallen below TEMPMIN.  If so, set up as a fault.
;
;  Tsens1 < TEMPMIN is our comparison
;
Chk_T1_Min:     cmpa    #TEMPMIN          ;Have we reached the min temp
                bcc     Chk_T2_Max        ;No, look at Sensor #2
                bset    ch_status, 82h    ;Update status word
                bra     Set_FaultMode     ;Now in fault mode

;  Check for Temperature sense 2 (Tsens2) against Temperature Maximum to see
;  if we've exceeded TEMPMAX.  If so, set up as a fault.
;
;  Tsens2 > TEMPMAX is our comparison

Chk_T2_Max:     ldaa    t2_filt           ;Load the filtered temperature
                cmpa    #TEMPMAX          ;Have we reached the max temp
                bcs     Chk_T2_Min        ;No, look at the min
                bset    ch_status, 12h    ;Update status word
                bra     Set_FaultMode     ;Now in fault mode

;  Check for Temperature sense 2 (Tsens2) against Temperature Minimum to see
;  if we've fallen below TEMPMIN.  If so, set up as a fault.
;
;  Tsens2 < TEMPMIN is our comparison
;
Chk_T2_Min:     cmpa    #TEMPMIN          ;Have we reached the min temp
                bcc     Chk_ToFault       ;No, look at next fault
                bset    ch_status, 08ah   ;Update status word
                bra     Set_FaultMode     ;Now in fault mode

;  Check for Time since power-up against Maximum allowable time to charge to
;  see if we're doing okay.  If not, set up as a fault.
;
;  check for Time since power up > TMAX
; 
Chk_Tofault:    ldaa    one_hour_count    ;See how many hours it's been
                cmpa    #TMAX             ;Has the max limit been reached
                bls     Chk_MaxAmpLim     ;No, check the Amphour Counter
                bset    ch_status, 22h    ;update status word
                bra     Set_FaultMode     ;We're now in fault mode

;  Check the amount of amp hours (using amp mins) applied to see if the maximum
;  limit has been exceed.  If so, set as a fault.
;
;  Check Tot_Amp_Mins > Max_Amp_Mins
;
Chk_MaxAmpLim:  ldd     Tot_Amp_Mins      ;See how many amp mins it's been
                cpd     #MAX_AMP_MINS     ;Has the max limit been reached
                bls     Chk_OtFault       ;No, check for next error
                bset    ch_status, 46h    ;update status word
                bra     Set_FaultMode     ;We're now in fault mode

;  Check the Charge Port temperature discrete (PA1) for an overheat or failure
;  condition.  If the discrete is set, we have a failure.
;
Chk_OtFault:    ldx     #REGBAS           ;Get the register base address
                brclr   PORTA,x, 02h, chk_fault_end   ;Looks clean
                bset    Ch_Status, 42h    ;update status word

;  This is where we enter Fault Mode based on a reported failure.
;
Set_FaultMode:  bsr     Fault_Mode        ;Put us into Fault Mode

;  This routine sets us up to exit the interrupt.
;
Chk_Fault_End:  ldx     #REGBAS           ;Offset to registers
                bclr    TFLG1,x,OC2_CLEAR ;Clear IRQ
                rti                       ;Return from the interrupt

;****************************************************************************


;****************************************************************************
;  Fault_Mode    19 Apr 95 by J.W. Hagen, HTSC
;  Description:  This routine controls the entry into Fault Mode.
;  Prog flow:    Linear.
;  Regs Altered: A
;  Global Read:  None.
;  Global Write: Power,Ch_Command
;  Passed value: None.
;  Return value: None.
;
;  This is where Fault Mode is entered based on a reported failure.
;
Fault_Mode:     ldaa    #MODE255          ;Enter Fault Mode
                staa    mode              ;Update the mode
                bset    Flag1,010h        ;Update mode transition flag
                bset    Flag4,040h        ;Show Fault Mode Entered
                clr     ch_command        ;request 0% pwr in fault mode
                clr     power             ;Power Msb = 0
                clr     power+1           ;Power Lsb = 0
                rts                       ;Back to the calling program

;****************************************************************************


;****************************************************************************
;  control       Scheduled to happen every  (6*32.77ms)=196msec
;  Description:  Controls output given a particular input. The algorithm
;                uses four modes: constant current #1, constant voltage #1,
;                constant current #2, & constant voltage #2.  At
;                the beginning of the cycle (large currents) the power is
;                controlled so that a constant charging current is applied.
;                After the charging current has relaxed (ITRIP) the power
;                shall be controlled so that a constant charging voltage is
;                applied.  After the voltage has increased to some max value,
;                VTRIP, during constant voltage mode, the previous two cycles
;                are repeated with different values for voltage & current trip
;                points.
;
;                Modified to slow down the approach to set points.  This will
;                prevent an overshoot which occurs when the customer removes
;                the paddle and immediately reinserts it.  10 May 95  JWH
;
;  Prog flow:    Called from tof_irq, linear execution. Call to read_ports
;  Regs Altered: x, a, b.
;  Global Read:  v_filt, i_filt, disc_status.
;  Global Write: ch_command, ch_status, ch_state
;  Return value: Results of this routine are J1850 data words.
;
;                 Moved I_COR & V_COR to Customer.H file.  11 Mar 95  JWH
;
;
;                    ===============================
;                    CRITICAL INTEGRATION PARAMETERS
;                    ===============================
;
;  When calculating trip points be sure and use appropriate scale factors that
;  are derived from the specific charger controller running the intended
;  firmware.
;
;  The current scale factor is derived in the following manner :
;
;                              1A  *  Rs(ohm)              255 lsb
;   current scale factor   =  ----------------- * 100 *  -----------
;                                    1A                      5  V
;
;     where Rs = current shunt resistor (Ohms)
;
;     For a 2 mOhm shunt resistor , current scale factor= 10.2 lsb/A
;
;  NOTE: The 10.2 lsb/A has been tweaked to 10.5.  This was accomplished using
;        a Fluke 79 with a current probe.  Also, this tweaking brings the
;        readings displayed in the diags program to approximately +/- 1 amp.
;
; The voltage scale factor is derived in the following manner :
 
;                               1V *  R3                255 lsb
;  voltage scale factor =     --------------------- *  ---------
;                               1V * (R1 + R2 + R3)       5  V
 
;        where R1,R2,R3 = three resistors comprising VBATT breakdown
;                         circuitry in voltage sense branch of charger
;                         controller.
 
;   For R1=R2= 432 KOhm  &  R3= 8.45 KOhm ,
;   voltage scale factor= .4940 lsb/V (nominal value)
;                       = .4928 ( 1st 2nd gen production unit )
;                       = .4983 Based on Fluke 79 and calibration for diags
;                               program.
;
; The following two parameters are the constant current & constant voltage
; gain parameters (I_COR & V_COR).  Two equations describe the dependence
; of these gain terms on various charging system parameters.  Specific
; charging system parameters yield a range throughout which charging system
; will be stable.  If desired gain falls outside range, system stability
; is in jeopardy.
;
;                               VBATT
; [1]     0  <  I_COR   <   -----------------
;                             10 * PMAX * Rs
;
;
;                            10 * VBATT * ( R1 + R2 + R3 )
; [2]     0  <  V_COR   <   --------------------------------
;                             PMAX * Rbatt * R3
;
;
;      where    VBATT = instantaneous traction battery voltage (VDC)
;               PMAX  = maximum charger power (W)
;               Rs    = current shunt resistor (Ohms)
;               R1,R2,R3 = three resistors comprising VBATT breakdown
;                          circuitry in voltage sense branch of charger
;                          controller.
;               Rbatt = traction battery impedance
 
; In the implementation of the original 2nd generation charging algorithm,
; the following assumptions were used :
;
;               VBATT = 400 VDC
;               PMAX  = 6600 W
;               Rs    = 2 mOhm
;               R1,R2,R3 = 432K, 432K, 8.45K (Ohm) respectively
;               Rbatt = 1 Ohm
 
; The above assumptions yield the following allowable ranges for
; I_COR & V_COR :
 
;               0 < I_COR <= 3    unscaled

;               0 < I_COR <= 48   scaled by 2^4
 
;               0 < V_COR <= 62   unscaled
;               0 < V_COR <= 992  scaled by 2^4
;               (maximum V_COR allowed is 255)
;
; When scaling for added gain control resolution use the upper limit
; of the allowable range for unscaled gain (max_gain_unscaled), obtained
; from either relation [1] or [2], and calculate scale factor (SF) under the
; following constraints :
;
;    [3]       max_gain_unscaled * SF --> 255      for highest resolution
;
;    [4]       SF = 2^N    for easily implementable scale down to ch_command
 
 
; Example :    let max_gain_unscaled = 3
;               then SF = 255/3 = 85
;
;                        SF = 2^N
;                   log(85) = N log(2)
;                         N = 6.4
;                   use   N = 6              to update integration parameter
;              therefore SF = 2^6 = 64
;
;              range is 0 < K < (2*64)
;                       0 < K < 128          define range for gain
 
;     overflow limit = 2^(8 + N)
;                    = 2^13
;                    = 4000 Hex
 
MODEDELAY_MAX:  equ     250                ;250 Second delay between modes

Control:        ldaa    v_raw             ;Get voltage
                staa    v_ctrl            ; to ensure const thru module
                staa    v_ctrl2           ; to ensure const thru module

                ldaa    i_filt            ;Get current
                staa    i_ctrl            ; to ensure const thru module
                ldy     #i_ctrl           ;point to ram loc for measured value

; Control starts here

                ldaa    Mode              ;What mode are we in at this time
                cmpa    #Mode1            ;Is it mode 1
                beq     Cv_Mode1          ;Yes, we're Constant Voltage #1
                cmpa    #Mode2            ;Is it mode 2
                beq     Cv_Mode2          ;Yes, we're Constant Voltage #2
                cmpa    #Mode3            ;Is it mode 3
                beq     Cv_Mode3          ;Yes, we're Constant Voltage #3
                cmpa    #Mode4            ;Is it mode 4
                beq     Cv_Mode4          ;Yes, we're Constant Voltage #4
                rts                       ;In Fault or Completion Mode where no
                                          ;  control is needed.  Exit

;  Select the address of the parameters for this mode
;
;  Mode 1 = Normal Constant Voltage
;
Cv_Mode1:       ldx     #mode1_parms      ;point to control param block
                brset   flag3,80h,Do_Ctrl ;If set, bypass trip point
                bra     Cv_Proceed        ;So we can press on

;  Mode 2 = Normal Constant Voltage
;
Cv_Mode2:       ldx     #mode2_parms      ;point to control param block
                brset   flag3,40h,Do_Ctrl ;If set, bypass trip point
                bra     Cv_Proceed        ;So we can press on

;  Mode 3 = Traction Pack Leveling Constant Voltage
;
Cv_Mode3:       ldx     #mode3_parms      ;point to control param block
                brset   flag3,20h,Do_Ctrl ;If set, bypass trip point
                bra     Cv_Proceed        ;So we can press on

;  Mode 4 = Auxiliary Battery Charging Constant Voltage
;
Cv_Mode4:       ldx     #mode4_parms      ;point to control param block
                brset   flag3,10h,Do_Ctrl ;If set, bypass trip point

;  Check measured value against trip point
;
Cv_Proceed:     ldaa    0,y               ;regA= i_ctrl
                cmpa    1,x               ;check against trip point
                bhi     Do_Ctrl           ;if i_ctrl > ITRIP do control
                                          ;else mode exit condition met

;  If this mode is set for 0 hours and 0 mins, no need to check for a delay.
;
                tst     0,x               ;Required hours this mode
                bne     Check_Delay       ;Required hours > 0
                tst     4,x               ;Required minutes this mode
                beq     Enter_New_Mode    ;Required hours & minutes = 0

;  We need to hold the mode at least MODEDELAY_MAX seconds.  This is required
;  for when a brownout occurs.  Only required when the time for a mode is NOT
;  set for zero hours and zero minutes.
;
Check_Delay:    ldaa    Mode_Delay        ;How long have we been in delay
                cmpa    #MODEDELAY_MAX    ;Have we waited long enough
                bcs     do_ctrl1          ;Not enough time yet, keep working

Enter_New_Mode: jsr     Select_Mode       ;Changing to the next mode
                rts                       ;Back to TOF_IRQ

Do_Ctrl:        clr     Mode_Delay        ;Reset the counter
Do_Ctrl1:       bsr     Ctrl              ;do ctrl

Control_End:    rts                       ;Back to TOF_IRQ
 
;****************************************************************************


;******************************************************************************
;  Complete Mode  Modified 14 Sep 95 by J.W. Hagen, HTSC
;  Description:   This routine controls the entry into Completion Mode.
;  Prog flow:     Linear.
;  Regs Altered:  A
;  Global Read:   None.
;  Global Write:  Ch_Command
;  Passed value:  None.
;  Return value:  A=Mode Number
;
;  This is where Completion Mode is entered.  Called from Control.
;
;  Update the charge complete cycle counter if this is the first entry.
;
Complete_Mode:  brset   flag4,080h,Complete_ModeB  ;If set, previous entry

;  This is the first entry.  Update the ee_cycle_cnt counter.
;
                sei
                jmp     Load_Program      ;Count the power-on cycle
Complete_ModeA: jsr     Ram_Init          ;ReInitialize RAM with parameter data
                cli                       ;Enable the interrupts

;  If the number of power-on cycles has been reset to LEVEL_AFTER, it's time
;  to perform a mandatory traction pack leveling cycle.  Force the controller
;  into Mode 3 as the next Mode.
;
                ldaa    ee_cycle_cnt      ;How many cycles remaining
                cmpa    #LEVEL_AFTER      ;Is it time to level?
                bne     Complete_ModeB    ;No, normal completion mode
                ldaa    #080h             ;Set the leveling bit
                staa    ch_state          ;Show the charger in leveling
                ldaa    #MODE3            ;Mandatory Traction Pack Leveling
                bra     Complete_ModeC    ;Save the updated mode

;  Process all requirements for charge complete.
;
Complete_ModeB: ldaa    #1                ;Select SCM message one
                staa    ch_status         ;Send message as charging complete
                ldaa    #100              ;For Charge State = 100%
                staa    ch_state          ;Update the charge state
                ldaa    #MODE254          ;Enter Completion Mode

Complete_ModeC: bset    flag4,080h        ;Show Completion Mode Entered
                bclr    flag4,031h        ;Clear the Leveling In Progress Flag
                                          ;Clear the Aux Battery Charging Flag
                                          ;Clear the Aux Battery Needs Charging
                clr     ch_command        ;Request 0% pwr in completion mode
                clr     power             ;Clear the Power Command MSByte
                clr     power+1           ;Clear the Power Command LSByte

Complete_Rts:   rts                       ;Back to the calling program

;****************************************************************************


;****************************************************************************
;  ctrl
;  Description:  Constant current/ constant voltage control algorithm.
;  Prog flow:    Called from control, linear execution.
;  Regs Altered: a, b.
;  Global Read:  v_ctrl, i_ctrl, mone_hour_count
;  Global Write: ch_command
;  Passed value: address of integration parameters block in reg X
;                address of v_ctrl or i_ctrl in reg Y
;  Return value: None.
;
;                Moved Slew Rate limiters to Customer.h file.  11 Mar 95  JWH
;
;  First check to see if we have exceeded our time limit in this mode before
;  we do anything else.
;
ctrl:           bclr    Flag1,08h         ;Reset the negative error flag
                ldaa    mone_hour_count   ;How many hours in this mode
                cmpa    0,x               ;check for timeout : hours
                bhi     enter_newmode     ;enter new mode if
                                          ;mode time(hrs) > timeout (hrs)
                blo     ctrl_proceed      ;proceed with ctrl if
                                          ;mode time(hrs) < timeout (hrs)

;  check minutes only when mode time (hrs) = timeout (hrs)
;
                ldaa    mone_min_count    ;How many minutes in this mode
                cmpa    4,x               ;check for timeout : minutes
                blo     ctrl_proceed      ;proceed with ctrl if
                                          ;mode time(mins) < timeout (mins)
 
;  enter new mode because mode exit condition met
;
Enter_Newmode:  jsr     Select_Mode       ;Changing to next mode
                rts                       ;Back to TOF_IRQ

;  We didn't time out so let's proceed
;
ctrl_proceed:   ldaa    2,x               ; regA = set point
                suba    2,y               ; regA = set point - measured val
                                          ;      = error (signed)

                staa    actual_error      ;Remember SetPoint - Measured

                bcc     Ctrl_Gm           ;If set point was > measured, branch
                bset    Flag1,08h         ;We had to borrow, set neg error flag
                                          ;(0=False 1=True)
                nega                      ;regA = set point - measured val
                                          ;     = error (unsigned)

                bclr    Flag3,08h         ;For positive error calculations

;  Gain control, how's it going
;
ctrl_gm:        ldab    3,x               ;regB = gain
                mul                       ;regD=error * gain
                std     ctrl_temp         ; ctrl_temp= error * gain (unsigned)

;  See if we were in the negative.  If so, gain was okay or we need less.  If
;  not, we need more.
;
                brclr   Flag1,08h,Ctrl_Pos ;If error >=0, then branch
 
;  error < 0 so we need less gain
;
;  Pn   = Power New
;  Pn-1 = Power Old
;
                ldaa    #200              ;Anything larger than sample period
                staa    sample_periods    ;To force an update when it goes pos

                ldaa    actual_error      ;SetPoint - Measured
                cmpa    #5                ;Is SetPoint - Measured within 5 lsb
                bcc     Ctrl_Neg2         ;No, normal calculation

;  Since SetPoint - Measured < 10 lsb's, start decrementing the new power
;  power command by 1.  Do this be changing the previously saved value from
;  SetPoint - Measured.  Force it to equal 1 bit.  This slows down the rate of
;  change by the power command.
;
                ldd     #1                ;Decrement power by 1 bit
                std     ctrl_temp         ;So SetPoint - Measured forced to = 1

;  Double the error
;
Ctrl_Neg1:      ldab    actual_error      ;Get SetPoint - Measured (unsigned)
                lslb                      ;Double the value of the neg error
                bcc     Ctrl_Neg1a        ;Doesn't exceed 255, branch
                ldab    #255              ;Load max value

Ctrl_Neg1a:     ldaa    3,x               ;Normal Gain Factor
                mul                       ;(((SetPoint-Measured)unsigned)2)gain
                std     ctrl_temp         ;Save as the amplified error

Ctrl_Neg2:      ldd     power             ;power = Pn-1
                subd    ctrl_temp         ;regD=Pn=Pn-1 - (error * gain)
                bcs     Ctrl_Neg3
                std     power             ;Save the updated power command
                cpd     #16               ;Is it at least .3% power
                bcc     scale_truncate    ;If power > ctrl_temp, scale

;  Pn-1 - (error*gain) < 0
;
Ctrl_Neg3:      clra                      ;Clear the power command MSByte
                clrb                      ;Clear the power command LSByte
                bra     Update_Power      ;Save as the updated charge command

;  error >= 0
;
Ctrl_Pos:       brclr   Flag3,08h,Ctrl_PosA   ;Had neg error, norm calculations
                inc     sample_periods    ;Increment the delay counter
                ldaa    sample_periods    ;Get the delay counter
                cmpa    #37               ;38 passes * .131msec = 4.978 secs
                bcc     Ctrl_PosA         ;Time to update the charge cmd

                ldd     power             ;Load old power command
                bra     Scale_Truncate    ;Process and save it

Ctrl_PosA:      clr     sample_periods    ;Clear the delay counter

;  Check the flag to see if a negative error has been experienced.  If so, no
;  limiters are in place to help prevent oscillations.
;
                brclr   Flag3,08h,Ctrl_Pos1a  ;Had neg error, calc norm

;  Limiters are in place.
;
                ldaa    actual_error      ;SetPoint - Measured
                cmpa    #10               ;Is SetPoint - Measured within 10 lsb
                bcc     Ctrl_Pos1         ;No, normal calculation

;  Since SetPoint - Measured < 10 lsb's, start incrementing the new power
;  power command by 1.  Do this be changing the previously saved value from
;  SetPoint - Measured.  Force it to equal 1 bit.  This slows down the rate of
;  change by the power command.
;
                ldd     #5                ;Increment power by 5 bits
                std     ctrl_temp         ;So SetPoint - Measured forced to = 5
                bra     Ctrl_Pos1a        ;

Ctrl_Pos1:      ldd     ctrl_temp         ;Get the amplified error
                cpd     #48               ;Limit to 1% power change
                bcs     Ctrl_Pos1a        ;Less than 48, it'll work
                ldd     #48               ;Load 1% power equivilant
                std     ctrl_temp         ;Save as 1% power delta

Ctrl_Pos1a:     bset    Flag3,08h         ;Return to restricted changes
                ldd     power             ; power = Pn-1
                addd    ctrl_temp         ; regD= Pn = Pn-1 + (error*gain)
                cpd     #OVERFLOW_LIMIT   ; look for overflow
                bhs     sat_hi            ; if overflow , then saturate hi

;  This is where we perform the scale prior to saving the data
;
Scale_Truncate: jsr     Current_Limit     ;Check the current limiter
                bsr     BrownOut          ;Check for a brownout
                std     power             ; update state variable
                ldy     #POWER_SCALE      ;regY= N , where scale factor = 2^N
                jsr     Shift_Right_D     ;Perform the necessary truncation
                bra     update_ch_cmd     ;We're done, time to save the data

;  This is where we start to force it into saturation
;
Sat_Hi:         ldd     #OVERFLOW_LIMIT   ;regD=16-bit 'power' overflow
                subd    #1                ;regD=max value, start lowering it
                jsr     Current_Limit     ;Check the current limiter

Update_Power:   bsr     BrownOut          ;Check for a brown out
                std     power             ;saturate

;  Update the Charge Command
;
Update_Ch_Cmd:  stab    ch_command        ;truncate ms byte of 16-bit of the
                                          ;'power' variable to get the
                                          ; 8-bit ch_command
                rts                       ;Back to Control

;****************************************************************************


;****************************************************************************
;  BrownOut      12 Dec 94 by J.W. Hagen, HTSC
;  Description:  This routine controls a predetermined timeout when a brownout
;                condition is in progress, loss of SCM J1850 transmitted msgs
;                or the SCM tripped / shut-down.  Also controls recovery from a
;                BrownOut and/or entry into fault mode should recovery not
;                occur within BrownOut_Lim seconds.
;  Prog flow:    Linear.
;  Regs Altered: A
;  Global Read:  Power
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
BrownOut_Max:   Equ     3                 ;Brownout in progress after 3 seconds
BrownOut_Lim:   Equ     30                ;Maximum # of seconds for brownout
BrownOut_Hold:  Equ     10                ;Maximum # of seconds to recover from
                                          ;a brownout before adjusting power.
BrownOut:       psha                      ;Save the A (New Power MSByte)
                pshb                      ;Save the B (New Power LSByte)

                ldaa    BrownOut_Secs     ;Compare to present value
                cmpa    #BrownOut_Max     ;Get the Maximum seconds counter
                blo     BrownOut2         ;Within limits, we're ok
                clr     BrownOut_Wait     ;Clear the BrownOut Restore Counter
                bclr    Flag1,020h        ;Clear SCM J1850 Tx Running OK flag
                bset    Flag1,040h        ;BrownOut Recovery Required Flag

;  See if it's been long enough.  If so, show as fault.  Error code is 016h.
;
                cmpa    #BrownOut_Lim     ;Have we been in brownout too long
                blo     BrownOut1         ;No, we're still under the limit
                bset    ch_status, 16h    ;update status word
                jsr     Fault_Mode        ;Enter Fault Mode

BrownOut1:      pulb                      ;Remove old Power LSByte
                pula                      ;Remove old Power MSByte
                ldd     power             ;Restore the Old Power Command (PN-1)
                rts                       ;Back to the calling program

BrownOut2:      brclr   Flag1,040h,BrownOut3  ;Not a previous BrownOut
                ldaa    #BrownOut_Hold    ;Get the required holding time
                suba    BrownOut_Wait     ;How long have we been in recovery
                bcc     BrownOut1         ;Not long enough, use old power level
                bclr    Flag1,040h        ;Clear previous brownout flag

BrownOut3:      clr     BrownOut_Wait     ;Clear the BrownOut Restore Counter
                pulb                      ;Restore New Power LSByte
                pula                      ;Restore New Power MSByte
BrownOut4:      rts                       ;Back to the calling program

;****************************************************************************


;****************************************************************************
;  Select Mode   02 Apr 96 by J.W. Hagen, HTSC
;  Description:  Selects the next Mode, based on the present Mode.
;  Prog flow:    Linear.
;  Regs Altered: A
;  Global Read:  Mode
;  Global Write: Mode
;  Passed value: None.
;  Return value: None.
;
Select_Mode:    ldaa    mode              ;Let's see what Mode we're in
                cmpa    #MODE1            ;Was it Mode 1
                beq     Select_Mode2      ;Yes, enter Mode 2

                cmpa    #MODE2            ;Was it Mode 2
                beq     Select_Mode254    ;Yes, enter Completion Mode

                cmpa    #MODE3            ;Was it Mode 3 (Leveling)
                beq     Select_Mode254    ;Yes, enter Completion Mode

                cmpa    #MODE4            ;Was it Mode 4 (Aux Battery Charging)
                beq     Select_Mode254    ;Yes, enter Completion Mode

                ldaa    #MODE255          ;Default is Fault Mode
                bra     Select_Rts        ;Save it and exit

Select_Mode2:   inca                      ;Changes to Mode 2
                bra     Select_Rts        ;Save it and exit

Select_Mode254: jsr     Complete_Mode     ;Process completion mode

Select_Rts:     staa    mode              ;Save as our new mode
                bset    Flag1,010h        ;Set Mode transition flag
                rts                       ;Finished here

;****************************************************************************


;****************************************************************************
;  Current_Limit 03 Apr 96 by J.W. Hagen, HTSC
;  Description:  This routine determines if the calculated power command will
;                cause current to exceed the value preset in the appropriate
;                Mode IMAX_MODEn setting.  If so, the old power command is
;                restored and used instead of the new calculation.
;  Prog flow:    Linear.
;  Regs Altered: D
;  Global Read:  Mode
;  Global Write: None.
;  Passed value: Power (newly calculated command).
;  Return value: Power (either old or original upon entry).
;
Current_Limit:  psha                      ;Save the A (New Power MSByte)
                pshb                      ;Save the B (New Power LSByte)

                ldab    i_raw             ;Use the raw current
                ldaa    mode              ;What mode is this
                cmpa    #MODE1            ;Is it Mode 1
                beq     Current_Mode1     ;Yes, check the max setting
                cmpa    #MODE2            ;Is it Mode 2
                beq     Current_Mode2     ;Yes, check the max setting
                cmpa    #MODE3            ;Is it Mode 3
                beq     Current_Mode3     ;Yes, check the max setting

;  Determine if the new power command will cause the Isense to exceed
;  IMAX_MODE4.
;
Current_Mode4:  cmpb    #IMAX_MODE4       ;How does it compare
                blo     Current_New       ;Looks okay, new power command valid
                bra     Current_Old       ;Too high, use old power command

;  Determine if the new power command will cause the Isense to exceed
;  IMAX_MODE3.
;
Current_Mode3:  cmpb    #IMAX_MODE3       ;How does it compare
                blo     Current_New       ;Looks okay, new power command valid
                bra     Current_Old       ;Too high, use old power command

;  Determine if the new power command will cause the Isense to exceed
;  IMAX_MODE2.
;
Current_Mode2:  cmpb    #IMAX_MODE2       ;How does it compare
                blo     Current_New       ;Looks okay, new power command valid
                bra     Current_Old       ;Too high, use old power command

;  Determine if the new power command will cause the Isense to exceed
;  IMAX_MODE1.
;
Current_Mode1:  cmpb    #IMAX_MODE1       ;How does it compare
                blo     Current_New       ;Looks okay, new power command valid

;  The new power command will cause the current to exceed the defined limit.
;  Restore the old power command and exit.
;
Current_Old:    pulb                      ;Remove old Power LSByte
                pula                      ;Remove old Power MSByte
                ldd     power             ;Restore the Old Power Command (PN-1)
                rts                       ;Back to the calling program

;  The new power command has been determined to be valid.  Restore the new
;  power command and exit.
;
Current_New:    pulb                      ;Restore New Power LSByte
                pula                      ;Restore New Power MSByte
                rts                       ;Back to the calling program

;****************************************************************************


;****************************************************************************
;  dlcs_irq      Modified 13 Dec 94 by JWH for detection of SCM xmit msgs
;
;  Description:  Data Link Controller Serial (DLCS) IRQ routine.  Called on:
;                1. An End Of Data (EOD) has been detected and CC byte is in
;                RFIFO.
;                2. The Receiver First In First Out (RFIFO) has 12 bits in it
;                and the 13th byte comes in
;                3. Tx in progress, no last byte, buffer half empty
;                4. DLCS wakeup (not implemented)
;                This routine checks for recieved data and empties the RFIFO
;                storing data to the rx_buffer, status to rx_status (single
;                buffered), and the completion code to rx_ccode.  It also
;                places the length of the currently recieved msg in rx_size.
;  Prog flow:    IRQ service.  Multiple calls to putdlcs.
;  Regs Altered: x, a, b.
;  Global Read:  None.
;  Global Write: rx_status, rx_buffer, rx_ccode, rx_size,
;                bad_ccode, fault_flag.
;  Return value: None.
;
DLC_DUMMY:      equ     00h               ;DLCS Dummy Data
DLC_RX_CMD:     equ     02h               ;DLCS Command:
                                          ;CMD: 0000,0010  Rx flush

dlcs_irq:       ldx     #rx_buffer        ;Load start address of buffer
                clr     rx_size           ;Clear message size
dlcs_irq1:      ldaa    #DLC_DUMMY        ;Load Data (Dummy)
                ldab    #DLC_RX_CMD       ;Recieve/flush command
                jsr     Putdlcs           ;Send message to DLCS
                staa    rx_status         ;Store status
                anda    #0e0h             ;AND stat w/ 1110,0000 Rx mask
                beq     dlcs_irq4         ;If not Rx info, go away
                cmpa    #0e0h             ;Check if CC only (last byte)
                beq     dlcs_irq2         ;Go store it and leave
                stab    0,x               ;Store data to buffer
                inx                       ;Bump buffer pointer
                inc     rx_size           ;Bump rx size data
                bra     dlcs_irq1         ;Go get more data

;  Since it was a receive message, check to see if there were any errors
;  detected.
;
dlcs_irq2:      stab    rx_ccode          ;Save completion code
;               andb    #80h              ;CC error bit mask
;               beq     dlcs_irq3         ;Beq if no error detected
;               ldaa    rx_ccode          ;Get completion code
;               staa    bad_ccode         ;Save faulty condition code

;  Determine if the receive message was from this node (Charge Controller) or
;  from another node (SCM).  If the Rx-Ccode has bits 4 & 5 set, then the
;  message originated from this node.  If not, then it was from another node.
;
dlcs_irq3:      ldaa    rx_ccode          ;Get the completion code
                anda    #30h              ;Transmitter successful (this node)
                cmpa    #30h              ;Is it from this node?
                beq     dlcs_irq4         ;Yes, it's not what we're looking for

;  Now that we know we have an actual message from the SCM, set the flag to
;  show the SCM J1850 is transmitting.  Then clear the brownout sec counter.
;
                bset    Flag1,020h        ;Set flag to show SCM J1850 Tx ok
                clr     BrownOut_Secs     ;Clear the J1850 Out, seconds counter

;  If this message is a "Wake Up" request, capture the max & min power
;  available from this charger.
;
;               ldx     #Rx_Buffer        ;Address of the J1850 receive buffer
;               ldaa    4,x               ;Get the J1850 Secondary ID
;               cmpa    #23h              ;Is it a wake up message
;               bne     dlcs_irq4         ;No, exit

;  This is a wake up request.  Obtain the power rating.
;
;               ldd     5,x               ;Charger Output Power Rating
;               std     SCM_Max_Power     ;Save as the maximum power
;               ldd     7,x               ;Charger Minimum Output Power Rating
;               std     SCM_Min_Power     ;Save as the maximum power
;               bset    Flag2,010h        ;Set to show SCM Output Pwr Known

dlcs_irq4:      rti                       ;Return from the interrupt

;****************************************************************************


;****************************************************************************
;  Temperature Compensated Voltage - 12 Mar 95 by J.W. Hagen, HTSC
;  Description:   Calculates voltage offsets based on temperature and applys
;                 these offsets to either V1, V2, Vtrip1 or Vtrip2, based on
;                 the present mode.
;  Prog flow:    Linear execution
;  Regs Altered: D,X,Y
;  Global Read:  V1, V2, Vtrip1, Vtrip2
;  Global Write: Modex_parms where x = 1, 2, 3 or 4
;  Passed value: None.
;  Return value: None.
;
;                       G A T E S   B A T T E R Y
;
;                    S E A L E D   L E A D   A C I D
;
;  The temperature compensation offset voltage is determined in the following
;  manner:
;
;     temp range      = -30 degree to +60 degree celsius (90 degree C spread)
;     where 1.28 lsb  = 2.6v * .4928 lsb/v = gassing voltage per cell at -30
;                       degrees celsius
;     where .0022 lsb = 4.44 mv = (.4 / 90) (slope of gassing voltage curve)
;                       also known as Temp_Comp_Mv which is preset in the
;                       customer.h file.  The .0022 used here is an example.
;     where 2.11 lsb  = per degree Kelvin
;     where Tn_Temp   = T1_filt or T2_filt (with TempMin already subtracted)
;     where 629 lsb   = 2.11 lsb * 298 Kelvin = 25 degrees C
;     where 513 lsb   = 2.11 lsb * 243 degrees kelvin (min temperature which
;                       is measurable by the temperature sensors.)
;     where 6         = Number of Cells per battery
;     where 27        = Number of Batteries in series
;     where 162       = TEMP_BATT_INFO = Number of Cells * Number of Batteries
;     where v/lsb     = volts per lsb (i.e. .4983 v/lsb)
;
;                    (4.44 mv * v/lsb)
;  - v_temp_offset = ----------------- * ((Tn_Temp + 513)-629) * TEMP_BATT_INFO
;                         2.11 lsb
;
;                      .002212452 lsb
;  - v_temp_offset =  ---------------  * (Tn_Temp - 116) * TEMP_BATT_INFO
;                         2.11 lsb
;
;  - v_temp_offset =  .001048555 lsb * (Tn_Temp - 116) * TEMP_BATT_INFO
;
;  - v_temp_offset =  .169865982 lsb * (Tn_Temp - 116)
;
;     Since we can't code fractions, we'll factor the entire fraction by 256
;     (2^8).   This allows us to take all numbers to integers.
;
;  - v_temp_offset =  43 * (Tn_Temp - 116)
;                    -----------------------
;                             256
;
;  With all of that said and done, let's look at an example.  We'll use a
;  temperature reading of 27 degrees celsius, 27 batteries and 6 cells.
;
;  The charge controller temperature sensors read voltage from 0 - 5 which
;  represents a temperature range of 245 K to 365 K.  Each lsb from the 5v
;  equals .019607843 volts (5v / 255 bits).  Since the temperature range has a
;  spread of 120 degrees, each lsb from the 5v equals .041666666 degrees Kelvin
;  (5v / 120 k).  For our example of 25 degrees C, our sensor will return a
;  value of 117 (decimal) which was determined as follows:
;
;     300 K = 27 degrees C
;     300 - 245 (temp min) = 55 K
;     55 K * .041666666 = 2.29166663 volts
;     2.29166663v / .019607843 v/lsb = 117 reading from the controller.
;
;  - v_temp_offset =    43 * (117 - 116)
;                      ------------------
;                             256
;
;  - v_temp_offset =    43 * 1
;                      --------
;                         256
;
;  - v_temp_offset = 43 / 256
;
;  - v_temp_offset = .16796875 
;
;
;  Local Variables and storage
;
TEMP_OFFSET:    equ     513               ;Min reading from temp sensor in lsb
TEMP_KELVIN:    equ     273               ;273 Kelvin = Zero Degrees C
TEMP_BATT_INFO: equ     {Batteries*Cells} ;Batteries * Cells
TEMP_CONSTANT1: equ     {{{{{TEMP_COMP_MV*VOLTS_SCALE}/LSB_PER_KELVIN}*TEMP_BATT_INFO}*256}/10000000}+1
TEMP_CONSTANT2: equ     {{{TEMP_NOMINAL_C+TEMP_KELVIN}*LSB_PER_KELVIN}/100}-TEMP_OFFSET

;  If the temperature compensation bit is set, perform temperature compensation
;
Temp_Comp:      brset   Flag1,01h,Temp_Comp1  ;Flag set, do temp comp
                rts                       ;Flag was off, return to main loop

Temp_Comp1:     bclr    Flag2,20h         ;Clear the Temp Comp Neg Offset Flag

;  Temp Comp is enabled, let's get started.  Look for the highest temperature.
;
                ldaa    t1_filt           ;Get the temperature for Sensor 1
                cmpa    t2_filt           ;How does it compare to Sensor 2
                bhs     Temp_Comp2        ;Sensor 1 has a higher temperature
                ldaa    t2_filt           ;Load the temperature for sensor 2

;  Next, subtract the 116 lsb's to get us to a known point, 25 degrees C.
;  If it's a negative value, set a flag so we'll know later.
;
Temp_Comp2:     suba    #temp_constant2   ;Tn_Temp - 116 offset
                bcc     Temp_Comp3        ;Result was not a negative number
                nega                      ;So we're with a positive number
                bset    Flag2,20h         ;Set a flag due to the negative #

Temp_Comp3:     ldab    #temp_constant1   ;Our multiplier
                mul                       ;43 * (Tn_Temp - 116)

;  We need to scale down to normalize the data.  Our temp_constant1 was scaled
;  by 2^8 (256) so it's time to normalize the offset so we can apply it.
;
                clc                       ;Clear the carry bit
                ldy     #08h              ;For our scaling factor
                jsr     Shift_Right_D     ;Divide by 256

;  The offset voltage is now in the B accumulator.  If it's a negative value,
;  when it's added to our Vset or Vn, it'll correct itself.  What we need to
;  know is if it was a negative number at one time.  If so, don't negate it.
;  If not, we will negate it because as temperature goes up, we want the offset
;  to be a negative offset.  Temperature goes down, our offset goes up.
;
                brset   Flag2,20h,Temp_Comp4
                negb                      ;Get the 2's compliment

Temp_Comp4:     stab    v_temp_comp       ;Save the LSByte, our offset voltage

;  So now we know what the voltage offset should be, let's determine the mode
;  we're in and calculate the trip or set point by adding the offset.
;
                ldaa    mode              ;Get the present mode
                cmpa    #Mode1            ;Mode 1
                beq     temp_v1           ;Yes, change the V1 setting
                cmpa    #Mode2            ;Mode 2
                beq     temp_v2           ;Yes, change the V2 setting
                cmpa    #Mode3            ;Mode 3
                beq     temp_v3           ;Yes, change the V3 setting
                cmpa    #Mode4            ;Mode 4
                beq     temp_v4           ;Yes, change the V4 setting
                rts                       ;We're outta here

;  Set up the Y register to point to the address of the voltage setting that
;  we'll be modifying.  Set the X register to the begining of the parameters
;  for this mode.  It will be used to look at the time required.
;
Temp_V1:        addb    #V1               ;Add in VTRIP1
                ldx     #mode1_parms      ;Starting address of this mode params
                ldy     #mode1_parms+2    ;Our destination Address
                bra     Temp_Comp5        ;Check the Hi/Low

Temp_V2:        addb    #V2               ;Add in the offset
                ldx     #mode2_parms      ;Starting address of this mode params
                ldy     #mode2_parms+2    ;Our destination Address
                bra     Temp_Comp5        ;Check the Hi/Low

Temp_V3:        addb    #V3               ;Add in the offset
                ldx     #mode3_parms      ;Starting address of this mode params
                ldy     #mode3_parms+2    ;Our destination Address
                bra     Temp_Comp5        ;Check the Hi/Low

Temp_V4:        addb    #V4               ;Add in the offset
                ldx     #mode4_parms      ;Starting address of this mode params
                ldy     #mode4_parms+2    ;Our destination Address

;  Now that we know what the voltage should be, we need to be sure it falls
;  within the TEMP_MAX_VOLTS and TEMP_MIN_VOLTS range.  The new value for our
;  voltage setting is presently in the B accumulator.  Leave it there until
;  we're sure it's a valid number.
;
Temp_Comp5:     cmpb    #Temp_Max_Volts   ;Compare to Maximum Allowed
                bls     Temp_Comp6        ;Looks okay here, check the min
                ldab    #Temp_Max_Volts   ;Get Maximum Allowed
                bra     Temp_Comp7        ;Now go check the mode time
Temp_Comp6:     cmpb    #Temp_Min_Volts   ;Compare to Minimum Allowed
                bhs     Temp_Comp7        ;Looks ok, we'll use the new value
                ldab    #Temp_Min_Volts   ;Get Minimum Allowed

;  If this mode is set for 0 hours and 0 mins, no need to modify the set point.
;
Temp_Comp7:     tst     0,x               ;Required hours this mode
                bne     Temp_Comp8        ;Required hours > 0, use new SetPoint
                tst     4,x               ;Required minutes this mode
                beq     Temp_Rts          ;Required hours & minutes = 0

;  We finally made it.  Save our voltage setting where it belongs and we're
;  back in business.
;
Temp_Comp8:     stab    0,y               ;Save as the new trip point

Temp_Rts:       rts                       ;We're outta here

;****************************************************************************


;****************************************************************************
;  Calculate State Of Charge - 14 Feb 95 by J.W. Hagen, HTSC
;  Description:  Calculates the approximate state of charge for the battery
;                pack.
;  Prog flow:    Linear execution
;  Regs Altered: D,X
;  Global Read:  ch_state
;  Global Write: ch_state
;  Passed value: None.
;  Return value: None.
;
;  Modified to track the number of amphours applied while in Mode 1.
;  14 Feb 95  JWH

;  Battery state of charge is calculated based on amp-hours, Vsense at the
;  start of the charging cycle, and Isense during the charging cycle.  The
;  following is our formula to determine the State of Charge (SOC):
;
;              (99) (Isense) (Ttotal)     Volt_Start - (11.58)(#Batt)
;     SOC   =  ----------------------- + -----------------------------
;                    (AH) (1.2)                 (1.2) (#Batt)
;  where:
;     99          = % Battery Charge Capacity
;     Isense      = Present Isense reading
;     Ttotal      = Total time elapsed since power-up (Modes 0 thru 4)
;     Volt_Start  = Vsense reading before entering Mode 1
;     11.58       = 0% State of Charge Open Circuit Voltage Reference Point
;     #Batt       = Number of Battery modules in series string
;     1.2         = 120% of Energy Returned
;     AH          = AmpHour capacity of Battery Pack
;     Zero_Ref    = 0% State of Charge Open Circuit Voltage Reference Point
;
;  Let's look at an example equation with the following known values:
;
;     Isense      = 20 amps
;     Ttotal      = .9 hours
;     Volt_Start  = 321 volts = 160 lsb / .4867 v/lsb
;     AH          = 60
;
;              (99) (Isense) (Ttotal)     (Volt_Start - (11.58)(#Batt)) * 100
;     SOC   =  ----------------------- + ------------------------------------
;                    (AH) (1.2)                       (1.2) (#Batt)
;
;  We'll invert the 99 on the left side of the equation to preclude a
;  requirement for 24 bit math due to the running total of Amp Minutes * 99
;  could exceed 65,535 (FFFFh).
;
;               (20) (.9)              (321 - (11.58 * 27)) * 100
;     SOC   =  -------------------- + ----------------------------
;               (60) (1.2 * .0101)               (1.2) (27)
;
;
;               (20) (.9)              (321 - (11.58 * 27)) * 100
;     SOC   =  -------------------- + ----------------------------
;               (60) (.012)                      (1.2) (27)
;
;
;                18      834
;     SOC   = ------- + ------
;               .72      32.4
;
;        NOTE: As you can see above, we now have a situation where we have
;              fractions involved in the equation.  During the actual execution
;              of the code, we'll be scaling up by a factor of 2^7 to preculde
;              working with fractions.
;
;     SOC   = 25.00 + 25.74
;
;     SOC   = 50%
;

Zero_Ref:       equ     {{{BATTERIES*1158}*VOLTS_SCALE}/VOLTS_MULTIPLY}/100
                                          ;(Number of Batteries*11.58)* V/lsb

Batt_Energy:    equ     {{{BATTERIES*120}*VOLTS_SCALE}/VOLTS_MULTIPLY}/100
                                          ;(Number of Batteries*1.20) * V/lsb

Amp_Energy:     equ     {AMP_HOURS*155}/100
                                          ;(AH * 1.2 * (1/99)) * 2^7
                                          ;  Scaled 2^7 for resolution

;  Perform calculation only if in Mode 1, 2, 3 or 4.
;
Calc_Soc:       ldaa    mode              ;Get the present mode
                cmpa    #MODE254          ;See where we're at
                bcc     Calc_Soc_Exit     ;Completion or Fault Mode

;  Next see if we are just starting a new minute.
;
                tst     one_sec_count     ;Are we at Zero seconds
                bne     Calc_Soc_Clr      ;No, then it's not time.  Clear flag.

;  Now test to see if we have already sampled for this minute.  If so, we'll
;  bail out and wait until the start of a new minute (seconds = 0).
;
                brset   flag1,02h, Calc_Soc_Exit ;If set, don't process

;  Let's perform the left side of the equation.  The multiplier is set to 100
;  which is the same value used to multiply the Current Scaling factor to pass
;  over to the diags program.  This then allows us to use the Scale_Current
;  variable to determine our actual measured amps at this time.
;
                ldaa    i_filt            ;Get the present current reading
                ldab    #AMPS_MULTIPLY    ;Same as used for Current scaling
                mul                       ;Scale it up
                ldx     #AMPS_SCALE       ;Get the factored Scale_Current
                idiv                      ;Converting to decimal Amps
                xgdx                      ;Isense actual amps is now in the B
                ldaa    #TSAMPLE          ;Our sample interval
                mul                       ;Yields Amps this sample period in B

;  At this point we know how many amps we're pulling.  Let's add it to the
;  running total.
;
                addd    tot_amp_mins      ;Add in previous Amp Minutes
                std     tot_amp_mins      ;Save the updated Amp Minutes

;  Calculate Amp Hours applied to charging the battery.
;
                ldx     #SAMPLES_HOUR     ;Sample periods per hour
                idiv                      ;(Tot_Amp_Mins/Periods Per Hour) =
                                          ;  AmpHours Applied to the Battery
                xgdx                      ;Move Amp Hours to the D Accumulator

;  The D accumulator contains the AmpHours applied to our battery pack since we
;  entered Mode 1.  The next step is to determine what percentage this is of
;  the battery capacity (based on 120% capacity).
;
;  In order to complete the left side of the equation we need to scale the Amp
;  Hours by 2^7.  This is because we have scaled the Amp_Energy by 2^7 as well.
;
                ldy     #7                ;Our Number of Left Shifts
                jsr     Shift_Left_D      ;Scale the Amp Hours up by 2^7
                ldx     #AMP_ENERGY       ;Our Amp_Hours * 1.2
                idiv                      ;Yields % increased since power-up
                xgdx                      ;Our % increase is now in the B

;  The left side of the equation is complete.  Now let's get the SOC at Startup
;
                brclr   Flag1,04h,Calc_Soc_1  ;Didn't start with - Soc
                cmpb    soc_at_start      ;See if we've met the deficit
                bls     Calc_Soc_5        ;Still in the negative, exit
                subb    soc_at_start      ;Subtract our deficit
                bra     Calc_Soc_2        ;Continue with the process

;  Now we'll add the right side of the equation to it and we're done.
;
Calc_Soc_1:     addb    soc_at_start      ;Get the original battery SOC
Calc_Soc_2:     cmpb    #99               ;How does it compare to 99%
                bhs     Calc_Soc_3        ;We're too high, save as 99%
                cmpb    #0                ;How are we compared to zero
                bhs     Calc_Soc_4        ;Looking good, save it
                clrb                      ;Clear the B accumulator
                bra     Calc_Soc_4        ;Save it and leave

Calc_Soc_3:     ldab    #99               ;Hard load a 99% value
Calc_Soc_4:     brset   flag4,01h,Calc_Soc_5 ;If leveling, bypass
                stab    ch_state          ;Our % State of Charge is in the B

;  Set the bit in the special flag byte, indicating the Calculation has been
;  completed for this time period.  Then exit the module.
;
Calc_Soc_5:     bset    Flag1,02h         ;Show the calculation complete
                rts

;  This flag is only cleared when we are on a second not equal to 00 (start of
;  a new minute).  Otherwise we leave the flag set to indicate the first pass
;  was completed and prevent sampling multiple times while the seconds counter
;  is still at 00 due to the micro-controller speed.
;
Calc_Soc_Clr:   bclr    Flag1,02h         ;Show the calculation as available
Calc_Soc_Exit:  rts

;****************************************************************************


;****************************************************************************
;  Shift Right, Double Accumulator - 21 June 95 by J.W. Hagen, HTSC
;  Description:  Shifts the contents of the Double Accumulator to the right by
;                the number contained in the Y register.
;  Prog flow:    Linear execution
;  Regs Altered: D,y
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Shift_Right_D:  lsrd                      ;Perform a right shift
                dey                       ;Show as another shift complete
                bne     Shift_Right_D     ;Loop till we're done
Shift_R_Exit:   rts                       ;Back to where we came from

;****************************************************************************


;****************************************************************************
;  Shift Left, Double Accumulator - 21 June 95 by J.W. Hagen, HTSC
;  Description:  Shifts the contents of the Double Accumulator to the left by
;                the number contained in the Y register.
;  Prog flow:    Linear execution
;  Regs Altered: D,X
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Shift_Left_D:   lsld                      ;Perform a left shift
                dey                       ;Show as another shift complete
                bne     Shift_Left_D      ;Loop till we're done
Shift_L_Exit:   rts                       ;Back to where we came from

;****************************************************************************


;****************************************************************************
;  Read_Ports    Modified 06 Jun 95  JWH
;  Description:  Reads Port A, B, C, & D.  Saves data to port_a_status,
;                port_b_status, port_c_status, and port_d_status.
;  Prog flow:    Linear execution, called by RTI IRQ, no local calls.
;  Regs Altered: a, x
;  Global Read:  None.
;  Global Write: port_a_status, port_b_status, port_c_status, port_d_status
;  Passed value: None.
;  Return value: REGBAS in the X
;
Read_Ports:     ldx     #REGBAS           ;Offset to registers

;  Process Port A
;
                ldaa    PORTA,x           ;All Discretes on PA0-PA7
                coma                      ;Invert Signals 16Feb93 rm
                staa    port_a_status     ;Set status bits

;  Process Port B
;
;               ldaa    PORTB,x           ;All Discretes on PB0-PB7
;               coma                      ;Invert Signals
;               staa    port_b_status     ;Set status bits

;  Process Port C
;
                ldaa    PORTC,x           ;All Discretes on PC0-PC7
                coma                      ;Invert Signals 16Feb93 rm
                staa    port_c_status     ;Set status bits

;  Process Port D
;
;               ldaa    PORTD,x           ;All Discretes on PD0-PD5
;               coma                      ;Invert Signals
;               staa    port_d_status     ;Set status bits

                rts                       ;Back to the calling program

;****************************************************************************


;******************************************************************************
;  Clear_PA7     Modified 12 Sep 95 by JWH
;  Description:  Clear the Discrete Output if we're in Completion or Fault
;                Mode.  This will then put to sleep any external connections
;                made to the discrete output.  The discrete will turn-off once
;                in Completion or Fault Mode at least Delay_PA7 seconds.
;  Prog flow:    Linear.
;  Regs Altered: A,X
;  Global Read:  PortA, Mode
;  Global Write: PortA, Bit 7
;  Passed value: None.
;  Return value: None.
;
DELAY_PA7:      equ     10                ;Set for a 10 second delay
;
Clear_PA7:      ldaa    mode              ;Let's see what Mode we're in
                cmpa    #MODE254          ;At least reached Completion Mode
                bcs     Clear_PA7_Rts     ;No, don't clear the D.O. yet

                ldaa    mone_sec_count    ;How many seconds have elapsed
                cmpa    #DELAY_PA7        ;See how we're doing with the delay
                blt     Clear_PA7_Rts     ;Not ready yet, press on

                ldx     #REGBAS           ;Offset to Registers
                bclr    PORTA,x,080h      ;Send it to the port
                bclr    flag4,04h         ;Show PA7 as turned off

Clear_PA7_Rts:  rts                       ;Our work is done!

;****************************************************************************


;****************************************************************************
;  putchar       modified 13 Sep 95  JWH
;  Description:  Sends an 8 bit ascii char to the SCI serial port.
;  Prog flow:    Linear, waits for TxDataRegEmpty SCI flag clear.
;  Regs Altered: None.
;  Global Read:  None.
;  Global Write: None.
;  Passed value: Data to send passed in register a.
;  Return value: None.
;
Putchar:        pshx                      ;store x
                ldx     #REGBAS           ;register offset
Putchar1:       brclr   SCSR,x,80h,putchar1  ;spin on tdre
                staa    SCDR,x            ;output char
                pulx                      ;restore x
                rts                       ;We're done here
 
;****************************************************************************


;****************************************************************************
;  Reset_Watchdog
;  Description:  resets watchdog timer, which times out every 1 sec.
;  Prog flow:    Linear execution
;  Regs Altered: COPRST,a
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Reset_Watchdog: psha                      ;Push the A on the stack
                pshx                      ;Push the X on the stack
                ldx     #REGBAS           ;Load the register base address
                ldaa    #055h             ;reset COP watchdog, 1st step of 2
                staa    COPRST,x          ;timer
                ldaa    #0aah             ;reset COP watchdog, 2nd step of 2
                staa    COPRST,x          ;
                pulx                      ;Restore the X from the stack
                pula                      ;Restore the A from the stack
                rts                       ;We're done

;****************************************************************************


;*****************************************************************************
;  Load Program  25 Mar 96 by J.W. Hagen, HTSC
;  Description:  This routine loads a program from EEProm and puts it in the
;                RAM area.  Then calls the program and executes it from RAM to
;                make changes in the EEProm.
;  Prog flow:    Linear.
;  Regs Altered: Y,X,A
;  Global Read:  Ram Data
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
Load_Program:   ldx     #Cycle_Counter    ;Starting Address of Prgm
                ldy     #0                ;Destination Address
Load_Program1:  ldaa    0,x               ;Get the next byte of data
                staa    0,y               ;Relocate to RAM storage
                inx                       ;Increment the counter
                iny                       ;Increment the counter
                cpx     #EndOfRamLoad     ;Have we finished the data
                blo     Load_Program1     ;Not enough, keep working
                jmp     sync_data         ;Execute from address 0x00

;*****************************************************************************


;*****************************************************************************
;  Cycle_Counter  25 Mar 96 by J.W. Hagen, HTSC
;  Description:   This routine counts the number of power-on cycles the
;                 charge controller experiences before requiring a battery
;                 leveling cycle.  The counter is preset to 0 when intially
;                 loaded into the EEProm.  Each cycle causes the counter to
;                 increment until reaching LEVEL_AFTER.  After reaching
;                 LEVEL_AFTER, the counter is cleared and the cycle is
;                 repeated.
;  Prog flow:     Linear.
;  Regs Altered:  X,D
;  Global Read:   Ram Data
;  Global Write:  None.
;  Passed value:  None.
;  Return value:  None.
;
;  Update the present count.
;
Cycle_Counter:  ldaa    cycle_ee_cnt      ;Number of cycles remaining
                deca                      ;Number of cycles remaining - 1
                beq     Cycle_Reset       ;If 0, time to reset the counter
                bra     Update_EEProm     ;Not enough cycles yet, update EEProm

;Reset the cycle counter to LEVEL_AFTER.  Comes here after leveling (Mode 3)
;  has just completed.
;
Cycle_Reset:    ldaa    #LEVEL_AFTER      ;Reset the cycle counter

;  Update the counter in the EEProm.
;
Update_EEProm:  ldx     #CYCLE_EE_CNT     ;EEProm Address to erase/write counter;
                bsr     ByteErase         ;Erase the byte
                bsr     ByteWrite         ;Save the changed value

;  Endless loop, we're done.
;
Update_Done:    jmp     Complete_ModeA    ;Back to the Main Program

;*****************************************************************************


;*****************************************************************************
;  Delay 10 ms   08 Dec 94 by J.W. Hagen, HTSC.
;  Description:  This routine creates a 10ms delay.
;  Prog flow:    Linear.
;  Regs Altered: X
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
;  3333 passes are needed for the loop.  This is based on the following:
;
;        3333 * 6 cycles * 500ns = 9.99ms
;
Delay10ms:      pshx                      ;Save the X register on the stack
                ldx     #3333             ;3333 passes through the loop
DelayLoop:      dex                       ;3 cycles - Decrement the counter
                bne     DelayLoop         ;3 cycles - Not at zero, keep looping
                pulx                      ;Restore the X register
                rts                       ;Back to the calling program

;*****************************************************************************


;*****************************************************************************
;  Byte Erase    08 Dec 94 by J.W. Hagen, HTSC.
;  Description:  This routine erases a byte of EEProm
;  Prog flow:    Linear.
;  Regs Altered: None.
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
;  Upon entry X register should contain the EEProm Address to be erased.
;
ByteErase:      ldab    #$16              ;0001 0100 PProg Byte Erase Mode
                stab    $103b             ;Put PProg register in Byte Erase
                stab    0,x               ;Save anything to do the erase
                ldab    #$17              ;0001 0111 PProg High Voltage Enable
                stab    $103b             ;Put PProg register in HV Mode
                bsr     Delay10ms         ;Perform 10ms delay
                clr     $103b             ;Turn off high voltage, set read mode
                rts                       ;Back to the calling program

;*****************************************************************************


;*****************************************************************************
;  Byte Write    08 Dec 94 by J.W. Hagen, HTSC.
;  Description:  This routine erases a byte of EEProm
;  Prog flow:    Linear.
;  Regs Altered: B
;  Global Read:  None.
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
;  Upon entry:
;              X register should contain the EEProm Address to be erased.
;              A accumulator should contain data to be programmed.
;
ByteWrite:      ldab    #$02              ;0000 0010 Set Pprog Eelat Bit
                stab    $103b             ;Put PProg register in Eelat Mode
                staa    0,x               ;Save new information
                ldab    #$03              ;0000 0011 PProg EELAT & EEPGM Enable
                stab    $103b             ;Put PProg register in Program Mode
                bsr     Delay10ms         ;Perform 10ms delay
                clr     $103b             ;Turn off bits, set read mode
                rts                       ;Back to the calling program
EndOfRamLoad:   equ     $                 ;

;*****************************************************************************


;****************************************************************************
;  Ram_Initialize - 19 Jun 95 by J.W. Hagen, HTSC
;  Description:  Load the data for sync bytes, program information, and the
;                data from the customer.h file into ram as required.
;  Prog flow:    Linear.
;  Regs Altered: Y,X,A
;  Global Read:  Ram Data
;  Global Write: None.
;  Passed value: None.
;  Return value: None.
;
;  We are placing the following data in RAM so we may send them out the port
;  and allow the diagnostic program to utilize the data for calculations and
;  displays to the user.
;
Ram_Init:       ldy     #Ram_Setup        ;Starting address of data to load
                ldx     #Sync_Data        ;Starting ram address to save data

Ram_Init1:      ldaa    0,y               ;Load the Ram data, starting at 00h
                staa    0,x               ;Save in parms location
                inx                       ;Increment our byte counter
                iny                       ;Increment the address offset
                cpy     #Ram_Last_Byte    ;Have we sent all the data yet
                blt     Ram_Init1         ;No, loop till we're done
                rts
;******************************************************************************


;*****************************************************************************
;  Ram_Out        03 Apr 96 by J.W. Hagen, HTSC
;  Description:   This routine sends all of the Ram data out the RS232 port.
;  Prog flow:     Linear.
;  Regs Altered:  Y,A
;  Global Read:   Ram Data
;  Global Write:  None.
;  Passed value:  None.
;  Return value:  None.
;
Ram_Out:        jsr     Reset_Watchdog    ;reset COP watchdog first
;
;  Send the Ram Data
;
                ldy     #0                ;Starting Address of Ram to Dump
Ram_Out1:       ldaa    0,y               ;Load the Ram data, starting at 00h
                jsr     Putchar           ;Send this byte
                iny                       ;Increment the address offset
                cpy     #256              ;Have we sent 256 bytes yet
                blt     Ram_Out1          ;No, loop till we're done
Ram_Rts:        rts                       ;We're done, bail out

;*****************************************************************************


;*****************************************************************************
;  Critical integration parameters control routine accesses.  Do NOT change the
;  order of this data.  It must coincide with the dianostic program.
;
Ram_Setup:      dfb     250               ;1st Sync Byte
                dfb     243               ;2nd Sync Byte
                dfb     32                ;3rd Sync Byte

                dfb     "Ev13.172"        ;Name of this program (xxxxxxxx.ASM)
                                          ;MUST always be 8 bytes.
                                          ;NEVER change this information!

                dfb     "041296"          ;Program Revision Date (mm/dd/yy)
                                          ;MUST always be 6 bytes

                dwm     VOLTS_MULTIPLY    ;Multiplier used for Voltage
                dwm     VOLTS_SCALE       ;Voltage Scaled * Voltage_Multiply

                dwm     AMPS_MULTIPLY     ;Multiplier used for Current
                dwm     AMPS_SCALE        ;Current Scaled * Current_Multiply

                dfb     TYPE_BATTERY      ;Code for Type of Battery
                dfb     STRINGS           ;Number of Battery Strings
                dfb     BATTERIES         ;Number of Batteries
                dfb     CELLS             ;Number of Cells per battery
                dfb     AMP_HOURS         ;Battery Ampere Rating
                dwm     MAX_AMP_MINS      ;Max Amp Mins allowed to be returned
                dfb     VMAX              ;Maximum Voltage
                dfb     VMIN              ;Minimum Voltage

                dfb     SAMPLES_HOUR      ;Number of Current samples per hour

                dfb     TEMPMAX           ;Maximum Temperature
                dfb     TEMPMIN           ;Minimum Temperature for Charging

mode1_cv:       dfb     T1_HOURS          ;Mode 1 maximum hours allowed
                dfb     ITRIP1            ;Voltage trip for mode 1
                dfb     V1                ;Set current for mode 1
                dfb     V_COR             ;Scaled current gain (Kcc)
                dfb     T1_MIN            ;Mode 1 maximum mins allowed
                                          ; (hours must be reached first)
 
mode2_cv:       dfb     T2_HOURS          ;Mode 2 maximum hours allowed
                dfb     ITRIP2            ;Current trip for mode 2
                dfb     V2                ;Set voltage for mode 2
                dfb     V_COR             ;Scaled voltage gain (Kcv)
                dfb     T2_MIN            ;Mode 2 maximum mins allowed
                                          ; (hours must be reached first)
 
mode3_cv:       dfb     T3_HOURS          ;Mode 3 maximum hours allowed
                dfb     ITRIP3            ;Current trip for mode 3
                dfb     V3                ;Set voltage for mode 3
                dfb     V_COR             ;Scaled voltage gain (Kcv)
                dfb     T3_MIN            ;Mode 3 maximum mins allowed
                                          ; (hours must be reached first)
 
mode4_cv:       dfb     T4_HOURS          ;Mode 4 maximum hours allowed
                dfb     ITRIP4            ;Current trip for mode 4
                dfb     V4                ;Set voltage for mode 4
                dfb     V_COR             ;Scaled voltage gain (Kcv)
                dfb     T4_MIN            ;Mode 4 maximum mins allowed
                                          ; (hours must be reached first)
 
                dfb     TMAX              ;Maximum Charging Time
                dfb     TEMP_MAX_VOLTS    ;Maximum Temp Compensated Voltage
                dfb     TEMP_MIN_VOLTS    ;Minimum Temp Compensated Voltage

CYCLE_EE_CNT:   dfb     LEVEL_AFTER       ;Preset number of power-on cycles
                dfb     LEVEL_AFTER       ;Perform leveling after n cycles

Ram_Last_Byte:  equ     $                 ;End of data being sent to Ram

;****************************************************************************


;****************************************************************************
;   Interrupt vector orgs:
                org     0ffdeh
                dwm     tof_irq           ;Timer Overflow
 
                org     0ffe6h
                dwm     oc2_irq           ;Output Compare 2 IRQ
 
                org     0ffe8h
                dwm     oc1_irq           ;Output Compare 1 IRQ
 
                org     0fff0h
                dwm     rti_irq           ;Real Time Interrupt
 
                org     0fff2h
                dwm     dlcs_irq          ;DLCS Interrupt (On *IRQ pin)
 
                org     0fffah            ;COP causes cold start
                dwm     RESET
 
                org     0fffeh            ;Reset Vector
                dwm     RESET
