;****************************************************************************
;
;                              E v 1 3 . 1 7 2
;
;                           R A M   E q u a t e s
;
;                       G l o b a l   V a r i a b l e s
;
;                             1 2   A p r   9 6
;
;****************************************************************************
;
;                             W  A  R  N  I  N  G
;
;  The Global Variables should not change in the sequence they are located in
;  RAM.  The user interface program and the RS232 port expect the variables to
;  be located in specific locations.  Should you decide to change the storage
;  locations, you MUST also modify the user interface (Ev.Exe program to
;  reflect these changes.  Failure to do so will not affect the operating logic
;  of this program but will affect the performance and operation of the RS232
;  interface with the user program.
;
;****************************************************************************
;
sync_data:      dfs     3                 ;000-002, RS232 Sync Bytes

;****************************************************************************
;  The following storage area is to provide the computer attached to the RS232
;  port diagnostic interface information concerning this program and it's
;  revision date.
;
program_id:     dfs     14                ;003-010 Program Name
                                          ;011-016 Revision Date
;****************************************************************************
;
;  This is the start of the global RAM variables.
;
;  This set is loaded from the .asm file upon startup of the controller.  Do
;  NOT change the order of this data without also changing the order in which
;  the data is decoded with the C program.
;
;  Scaling and multiplication factors for converting amps and voltage.
;
multiply_volts: dfs     2                 ;017-018 Scaled Voltage Multiplier
scale_volts:    dfs     2                 ;019-020 Voltage Scaling factor

multiply_amps:  dfs     2                 ;021-022 Scaled Amps Multiplier
scale_amps:     dfs     2                 ;023-024 Amps Scaling factor


;  Battery Information
;
battery_type:   dfs     1                 ;025     Code for Battery Type
string:         dfs     1                 ;026     Number of Battery Strings
battery:        dfs     1                 ;027     Number Batteries per string
cell:           dfs     1                 ;028     Number cells per battery
amp_capacity:   dfs     1                 ;029     Ampere capacity of batt pack
amp_mins_max:   dfs     2                 ;030-031 Max AmpMins to return
v_max:          dfs     1                 ;032     Maximum Voltage
v_min:          dfs     1                 ;033     Minimum Voltage
sample_perhour: dfs     1                 ;034     # of AmpHr Samples per hour

max_temp:       dfs     1                 ;035     Maximum Battery Temperature
min_temp:       dfs     1                 ;036     Minimum Battery Temperature

;****************************************************************************
;
;  Storage for mode parameters.
;
mode1_parms:    dfs     5                 ;037-041 All of Mode 1 parameters
mode2_parms:    dfs     5                 ;042-046 All of Mode 2 parameters
mode3_parms:    dfs     5                 ;047-051 All of Mode 3 parameters
mode4_parms:    dfs     5                 ;052-056 All of Mode 4 parameters

max_time:       dfs     1                 ;057     Maximum Time for charging
comp_max_volt:  dfs     1                 ;058     Max Temp Comp Voltage
comp_min_volt:  dfs     1                 ;059     Min Temp Comp Voltage
ee_cycle_cnt:   dfs     1                 ;060     Number of power-on cycles
levels_after:   dfs     1                 ;061     Level after N cycles

spare_bytes:    dfs     8                 ;062-069
;
;  End of EEPROM loaded ram data
;******************************************************************************
v_conv_box:     dfs     1                 ;070     Conversion Box Temp Sensor
v_ch_port:      dfs     1                 ;071     Charge Port Temp Sensor
mode_delay:     dfs     1                 ;072
sample_periods: dfs     1                 ;073
spare_byte2:    dfs     4                 ;074-077 Spare Bytes
actual_Error:   dfs     1                 ;078     SetPoint - Measured
ctrl_temp:      dfs     2                 ;079-080 holds 16-bit multiplicand of
                                          ;        error * gain in control routine
v_temp_comp:    dfs     1                 ;081     Temp Comp Voltage Offset


power:          dfs     2                 ;082-083 Updated Power Command
scm_max_power:  dfs     2                 ;084-085 SCM Max Power Rating
scm_min_power:  dfs     2                 ;086-087 SCM Min Power Rating
mode:           dfs     1                 ;088     Mode state variable

ch_command:     dfs     1                 ;089     J1850 msg data: 0 - 100% pwr request
ch_status:      dfs     1                 ;090     J1850 msg data: Charging status
ch_state:       dfs     1                 ;091     J1850 msg data: State of charge

bad_ccode:      dfs     1                 ;092     J1850 Bad Completion Code

v_raw:          dfs     1                 ;093     Raw voltage value
v_filt:         dfs     2                 ;094-095 Filtered/scaled voltage value
v_at_start_up:  dfs     1                 ;096     Batt Voltage Prior to Mode 1
v_auxbattery:   dfs     1                 ;097     Aux Batt A/D Voltage (AN4)
v_precharge:    dfs     1                 ;098     Dolphin Precharge A/D (AN5)

i_raw:          dfs     1                 ;099     Raw current value
i_filt:         dfs     2                 ;100-101 Filtered/scaled current value

t1_raw:         dfs     1                 ;102     Raw temperature 1 value
t1_filt:        dfs     2                 ;103-104 Filtered temp 1

t2_raw:         dfs     1                 ;105     Raw temperature 2 value
t2_filt:        dfs     2                 ;106-107 Filtered temp 2

tof_cnt:        dfs     1                 ;108     Timer IRQ Pass Counter
soc_at_start:   dfs     1                 ;109     State of Charge at Startup

;******************************************************************************
;  Real Time Clock
;
one_sec_count:    dfs   1                 ;110     1 tick = 1 sec (sys clock)
four_msec_count:  dfs   1                 ;111     1 tick = 4.1ms (sys clock)
one_min_count:    dfs   1                 ;112     1 tick = 1 min (sys clock)
one_hour_count:   dfs   1                 ;113     1 tick = 1 hour (sys clock)

;******************************************************************************
;  Mode Real Time Clock
;
mone_sec_count:   dfs   1                 ;114     1 tick = 1 sec (mode clock)
mfour_msec_count: dfs   1                 ;115     1 tick = 4.1ms (mode clock)
mone_min_count:   dfs   1                 ;116     1 tick = 1 min (mode clock)
mone_hour_count:  dfs   1                 ;117     1 tick = 1 hour (mode clock)

;******************************************************************************
;  J1850 Variables
;
rx_status:      dfs     1                 ;118     DLCS status byte on recieve
rx_size:        dfs     1                 ;119     DLCS current rx msg size in bytes
rx_ccode:       dfs     1                 ;120     DLCS completion code byte
rx_buffer:      dfs    20                 ;121-140 DLCS data recieve buffer
brownout_secs:  dfs     1                 ;141     BrownOut Seconds Counter
brownout_wait:  dfs     1                 ;142     BrownOut Recovery Delay

;****************************************************************************
;  Data preloaded by the asm program.  DO NOT change the order of this data!
;  Storage for other preloaded variables.
;
tot_amp_mins:   dfs     2                 ;143-144 Total of Amp Minutes

;****************************************************************************
;  The next variables are used to track the port status
;
port_a_status:  dfs     1                 ;145     Port A Discrete Status
port_b_status:  dfs     1                 ;146     Port B Discrete Status
port_c_status:  dfs     1                 ;147     Port C Discrete Status
port_d_status:  dfs     1                 ;148     Port D Discrete Status

bad_status:     dfs     1                 ;149     J1850 Bad Status

;******************************************************************************
; voltage iir  structure follows, note the following block of variables must be
; grouped together

vx:             dfs     1                 ;150     voltage iir x input
vb:             dfs     1                 ;151     voltage iir b0 coefficient
vbx:            dfs     2                 ;152-153 16 bit unsigned result of x * b
vw:             dfs     3                 ;154-156 24 bit state variable
va:             dfs     1                 ;157     voltage iir a1 coefficient
vy:             dfs     2                 ;158-159 16 bit voltage iir y output

; current iir  structure follows, note the following block of variables must be
; grouped together
 
ix:             dfs     1                 ;160     current iir x input
ib:             dfs     1                 ;161     current iir b0 coefficient
ibx:            dfs     2                 ;162-163 16 bit unsigned result of x * b
iw:             dfs     3                 ;164-166 24 bit state variable
ia:             dfs     1                 ;167     current iir a1 coefficient
iy:             dfs     2                 ;168-169 16 bit current iir y output

; temp1 iir  structure follows note, the following block of variables must be
; grouped together

t1x:            dfs     1                 ;170     temp1 iir x input
t1b:            dfs     1                 ;171     temp1 iir b0 coefficient
t1bx:           dfs     2                 ;172-173 16 bit unsigned result of x * b
t1w:            dfs     3                 ;174-176 24 bit state variable
t1a:            dfs     1                 ;177     temp1 iir a1 coefficient
t1y:            dfs     2                 ;178-179 16 bit current iir y output

; temp2 iir  structure follows note, the following block of variables must be
; grouped together

t2x:            dfs     1                 ;180     temp2 iir x input
t2b:            dfs     1                 ;181     temp2 iir b0 coefficient
t2bx:           dfs     2                 ;182-183 16 bit unsigned result of x * b
t2w:            dfs     3                 ;184-186 24 bit state variable
t2a:            dfs     1                 ;187     temp2 iir a1 coefficient
t2y:            dfs     2                 ;188-189 16 bit current iir y output

;****************************************************************************
;  Special Use Flags.  If the Bit is set (1), the function is enabled.
;
flag1:          dfs     1                 ;190     Special Purpose Flags #1
                                          ;        Bit 0= Temperature Comp.
                                          ;        Bit 1= Calc_Soc_Done
                                          ;        Bit 2= Calc_Soc_Start Negate
                                          ;        Bit 3= Gain Neg Error Flag
                                          ;        Bit 4= Mode Transition
                                          ;        Bit 5= SCM J1850 Transmitter
                                          ;        Bit 6= BrownOut In Progress
                                          ;        Bit 7= Spare

flag2:          dfs     1                 ;191     Special Purpose Flags #2
                                          ;        Bit 0= AmpHr_Tracking Flag
                                          ;        Bit 1= Monitor Temp Sensors
                                          ;        Bit 2= Monitor Dolphin Unit
                                          ;        Bit 3= Monitor Aux Battery
                                          ;        Bit 4= SCM Pwr Output Known
                                          ;        Bit 5= Temp Comp Offset Neg
                                          ;        Bit 6= Spare
                                          ;        Bit 7= Oc1_Irq_Pass (A/D)

flag3:          dfs     1                 ;192     Special Purpose Flags #3
                                          ;        Bit 0= Connection Present #1
                                          ;        Bit 1= Connection Present #2
                                          ;        Bit 2= Spare
                                          ;        Bit 3= Restrict Change Rate
                                          ;        Bit 4= Bypass Mode 4 Trip Pt
                                          ;        Bit 5= Bypass Mode 3 Trip Pt
                                          ;        Bit 6= Bypass Mode 2 Trip Pt
                                          ;        Bit 7= Bypass Mode 1 Trip Pt

flag4:          dfs     1                 ;193     Special Purpose Flags #4
                                          ;        Bit 0= Leveling Mode Entered
                                          ;        Bit 1= Level Voltage Tested
                                          ;        Bit 2= Dolphin Turned On
                                          ;        Bit 3= Leveling Enabled
                                          ;        Bit 4= Aux Battery Charging
                                          ;        Bit 5= Aub Batt Needs Charge
                                          ;        Bit 6= Fault Mode Entered
                                          ;        Bit 7= Completion Mode Entry

                                          ;        The following 3 variables
                                          ;        must be defined sequentially
v_ctrl:         dfs     2                 ;194-195 Hold voltage during Control
i_ctrl:         dfs     2                 ;196-197 Hold current during Control
v_ctrl2:        dfs     2                 ;198-199 Hold voltage during Control

level_entry:    dfs     1                 ;200     Number of times leveling
aux_batt_entry: dfs     1                 ;201     Aux Batt Charging Cycles

;****************************************************************************
;
;                       E N D   O F   R A M   E Q U A T E S
;
;****************************************************************************

